CKEDITOR.dialog.add('semwidgConfigDialog', function (editor) {

	var dialog = {};

	dialog.title = 'SemwidgJS Config';
	dialog.minWidth = 800;
	dialog.minHeight = 400;

	dialog.contents = [];

	var configTab = {};
	configTab.id = 'configProperties';
	configTab.label = 'Config Properties';

	configTab.elements = [
		{
			type: 'html',
			id: 'configTabHtmlInfo',
			html: '<div><h1 id="semwidgckConfigTabTitle">Enter Config Properties:</h1></div>'
		},
		{
			type: 'html',
			id: 'configTabHtmlForm',
			html: '<div id="semwidgckConfigTab" style="height: 360px; overflow: auto; padding: 2px; position: relative;"></div>',
			setup: function(widget) {
				CKEDITOR.SemwidgJS.SemwidgConfigDialog.setupConfigTab(widget);
			},
			commit: function(widget) {
				CKEDITOR.SemwidgJS.SemwidgConfigDialog.commitConfigTab(widget, editor);
			}
		}
	];

	dialog.contents.push(configTab);

	dialog.onShow = function() {
		if (CKEDITOR.SemwidgJS.data.configType && CKEDITOR.SemwidgJS.data.configType.lastIndexOf('config_', 0) === 0) {
			this.semwidgConfigType = CKEDITOR.SemwidgJS.data.configType.substr(7);
			CKEDITOR.SemwidgJS.SemwidgConfigDialog.buildConfigDialog(this, editor);

			CKEDITOR.SemwidgJS.SemwidgConfigDialog.setDefaultConfigData(this.semwidgConfigType);
		}
	};

	dialog.onOk = function() {
		this.semwidgClosingType = 'ok';
	};

	dialog.onCancel = function() {
		this.semwidgClosingType = 'cancel';
	};

	dialog.onHide = function() {
		if (this.semwidgClosingType == 'ok') {
			editor.setMode('source');
			editor.setMode('wysiwyg');
		}
	}

	return dialog;
});

CKEDITOR.SemwidgJS.SemwidgConfigDialog = function() {

};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.buildConfigDialog = function(dialog, editor) {
	var type = dialog.semwidgConfigType;

	var definition = dialog.definition;
	var configTab = definition.getContents('configProperties');

	// build Config Tab
	$('#semwidgckConfigTabTitle').text('Enter Config Properties for ' + editor.getSemwidgJS().Util.String.toUpperCamelCase(type) + ':');

	var configTab = $('#semwidgckConfigTab');

	var attributes = CKEDITOR.SemwidgJS.getConfigProperties(type);

	var configTabHtml = '';
	configTabHtml += '<table style="width: 100%;">';
	for (var i = 0; i < attributes.length; i++) {
		var attr = attributes[i];
		configTabHtml += '<tr>';
			configTabHtml += '<td style="padding: 4px 0;">' + editor.getSemwidgJS().Util.String.toUpperCamelCase(attr) + ':</td>';
			configTabHtml += '<td style="padding: 4px 0px 4px 8px; width: 100%;">';
			configTabHtml += '<div ';
			configTabHtml += 'id="semwidgckConfigProperty_input_' + attr + '" ';
			configTabHtml += 'class="semwidg-autosuggest-input" ';
			configTabHtml += 'contentEditable="true">&nbsp;</div>';
		configTabHtml += '</td>';
		configTabHtml += '</tr>';
	}
	configTabHtml += '</table>';

	configTab.html(configTabHtml);

	var inputs = configTab.find('.semwidg-autosuggest-input');
	CKEDITOR.SemwidgJS.SemwidgConfigDialog.buildAutosuggest(inputs, editor);
};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.buildAutosuggest = function(inputs, editor) {

	var id;
	var type;
	var autosuggest;
	for (var i = 0; i < inputs.length; i++) {
		id = $(inputs[i]).attr('id');
		type = id.substr(30).toLowerCase();

		switch(type) {
			case 'endpoint':
				autosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id, true);
				autosuggest.setRequestFunction(CKEDITOR.SemwidgJS.SemwidgConfigDialog.RequestFunction.endpoint(autosuggest, editor));
				autosuggest.setSelectFunction(CKEDITOR.SemwidgJS.SemwidgConfigDialog.SelectFunction.defaultSelectFunction(autosuggest, editor));
			break;
			case 'proxy':
				autosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id, true);
				autosuggest.setRequestFunction(CKEDITOR.SemwidgJS.SemwidgConfigDialog.RequestFunction.proxy(autosuggest, editor));
				autosuggest.setSelectFunction(CKEDITOR.SemwidgJS.SemwidgConfigDialog.SelectFunction.defaultSelectFunction(autosuggest, editor));
			break;
			case 'default':
				autosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id, true);
				autosuggest.setRequestFunction(CKEDITOR.SemwidgJS.SemwidgConfigDialog.RequestFunction.default(autosuggest, editor));
				autosuggest.setSelectFunction(CKEDITOR.SemwidgJS.SemwidgConfigDialog.SelectFunction.defaultSelectFunction(autosuggest, editor));
			break;
			case 'http-method':
				autosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id, true);
				autosuggest.setRequestFunction(CKEDITOR.SemwidgJS.SemwidgConfigDialog.RequestFunction.httpMethod(autosuggest, editor));
				autosuggest.setSelectFunction(CKEDITOR.SemwidgJS.SemwidgConfigDialog.SelectFunction.defaultSelectFunction(autosuggest, editor));
			break;
		}
	}
};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.setupConfigTab = function(widget) {
	var type = widget.element.getAttribute('data-sw-config');

	if(!type) {
		return;
	}

	var attributes = CKEDITOR.SemwidgJS.getConfigProperties(type);
	var attributeName;
	var value;
	for (var i = 0; i < attributes.length; i++) {
		attributeName = attributes[i];
		value = widget.element.getAttribute('data-swp-' + attributeName);
		if (value){
			$('#semwidgckConfigProperty_input_' + attributeName).text(value);
		}
	}

};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.commitConfigTab = function(widget, editor) {
	var type = widget.element.getAttribute('data-sw-config');

	if(!type) {
		return;
	}

	// add Id if not set
	if (editor.getSemwidgJS().Util.String.isEmpty($('#semwidgckConfigProperty_input_id').text())) {
		$('#semwidgckConfigProperty_input_id').text(type + '-' + editor.getSemwidgJS().Util.getUniqueId());
	}

	var attributes = CKEDITOR.SemwidgJS.getConfigProperties(type);
	var propertyName;
	var value;
	for (var i = 0; i < attributes.length; i++) {
		propertyName = attributes[i];
		value = $('#semwidgckConfigProperty_input_' + propertyName).text().trim();

		var old = widget.element.getAttribute('data-swp-' + propertyName);
		if (old !== value) {
			widget.element.setAttribute('data-swp-' + propertyName, value);
		}
	}

	CKEDITOR.SemwidgJS.newElementIds.push($('#semwidgckConfigProperty_input_id').text());
};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.setDefaultConfigData = function(type) {
	if (type && ('' + type).toLowerCase() == 'endpoint') {
		var method = $('#semwidgckConfigProperty_input_http-method');
		if (method.val() == null || method.val() == '') {
			method.val('POST');
		}
	}
};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.RequestFunction = function() {};
CKEDITOR.SemwidgJS.SemwidgConfigDialog.SelectFunction = function() {};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.SelectFunction.defaultSelectFunction = function(autosuggest, editor) {
	return function(value) {
		if (value && value != '') {
			autosuggest.element.text(value);
		}
	};
};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.RequestFunction.endpoint = function(autosuggest, editor) {
	return function(value) {
		var endpoints = editor.getSemwidgJS().getConfig().getEndpoints();
		var resultContains = [];
		var resultContainsNot = [];

		if (endpoints) {
			for (var i = 0; i < endpoints.length; i++) {
				if (endpoints[i].getId().indexOf(value) > -1) {
					resultContains.push(endpoints[i].getId());
				}else {
					resultContainsNot.push(endpoints[i].getId());
				}
			}
		};

		var simpleStringArraySort = function(a, b) {
			var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a, value);
			var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b, value);

			return levA - levB;
		};

		resultContains.sort(simpleStringArraySort);
		resultContainsNot.sort(simpleStringArraySort);

		var result = resultContains.concat(resultContainsNot);

		for (var i = 0; i < result.length; i++) {
			result[i] = ['<span style="font-family: monospace;">' + result[i] + '</span>', result[i]];
		}

		autosuggest.setResult(result);
	};
};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.RequestFunction.httpMethod = function(autosuggest, editor) {
	return function(value) {
		var methods = ['POST', 'GET'];
		var resultContains = [];
		var resultContainsNot = [];

		if (methods) {
			for (var i = 0; i < methods.length; i++) {
				if (methods[i].indexOf(value) > -1) {
					resultContains.push(methods[i]);
				}else {
					resultContainsNot.push(methods[i]);
				}
			}
		};

		var simpleStringArraySort = function(a, b) {
			var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a, value);
			var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b, value);

			return levA - levB;
		};

		resultContains.sort(simpleStringArraySort);
		resultContainsNot.sort(simpleStringArraySort);

		var result = resultContains.concat(resultContainsNot);

		for (var i = 0; i < result.length; i++) {
			result[i] = ['<span style="font-family: monospace;">' + result[i] + '</span>', result[i]];
		}

		autosuggest.setResult(result);
	};
};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.RequestFunction.default = function(autosuggest, editor) {
	return function(value) {
		var defaults = ['TRUE', 'FALSE'];
		var resultContains = [];
		var resultContainsNot = [];

		if (defaults) {
			for (var i = 0; i < defaults.length; i++) {
				if (defaults[i].indexOf(value) > -1) {
					resultContains.push('' + defaults[i]);
				}else {
					resultContainsNot.push('' + defaults[i]);
				}
			}
		};

		var simpleStringArraySort = function(a, b) {
			var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a, value);
			var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b, value);

			return levA - levB;
		};

		resultContains.sort(simpleStringArraySort);
		resultContainsNot.sort(simpleStringArraySort);

		var result = resultContains.concat(resultContainsNot);

		for (var i = 0; i < result.length; i++) {
			result[i] = ['<span style="font-family: monospace;">' + result[i] + '</span>', result[i]];
		}

		autosuggest.setResult(result);
	};
};

CKEDITOR.SemwidgJS.SemwidgConfigDialog.RequestFunction.proxy = function(autosuggest, editor) {
	return function(value) {
		var proxies = editor.getSemwidgJS().getConfig().getProxies().slice();

		proxies.push({getId: function() {return 'NONE'}});
		proxies.push({getId: function() {return 'DEFAULT'}});

		var resultContains = [];
		var resultContainsNot = [];

		if (proxies) {
			for (var i = 0; i < proxies.length; i++) {
				if (proxies[i].getId().indexOf(value) > -1) {
					resultContains.push(proxies[i].getId());
				}else {
					resultContainsNot.push(proxies[i].getId());
				}
			}
		};

		var simpleStringArraySort = function(a, b) {
			var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a, value);
			var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b, value);

			return levA - levB;
		};

		resultContains.sort(simpleStringArraySort);
		resultContainsNot.sort(simpleStringArraySort);
		
		var result = resultContains.concat(resultContainsNot);

		for (var i = 0; i < result.length; i++) {
			result[i] = ['<span style="font-family: monospace;">' + result[i] + '</span>', result[i]];
		}

		autosuggest.setResult(result);
	};
};