CKEDITOR.dialog.add('semwidgConfigWizardDialog', function (editor) {

	var dialog = {};

	dialog.swCurrentStep = 0;
	dialog.swEndpoints;

	dialog.title = 'SemwidgJS Config Wizard';
	dialog.minWidth = 800;
	dialog.minHeight = 400;

	dialog.contents = [];

	var endpointTab = {};
	endpointTab.id = 'endpointTab';
	endpointTab.label = 'Endpoints';

	endpointTab.elements = [
		{
			type: 'select',
			id: 'predefinedSparqlEndpoints',
			label: 'Select your SPARQL endpoint',
			items: CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.getDefaultSparqlEndpoints(),
			inputStyle: 'width: 200px !important;',
			onChange: function(event) {

				var id = event.data.value;
				var item;
				for (var i = 0; i < this.items.length; i++){
					if (this.items[i][1] == id) {
						item = this.items[i];
						break;
					}
				}

				// fill input
				$('#semwidgckConfigWizardProperty_input_endpointId').val(item[1]);
				$('#semwidgckConfigWizardProperty_input_endpointUrl').val(item[2]);
				$('#semwidgckConfigWizardProperty_input_graphUri').val(item[3]);
				$('#semwidgckConfigWizardProperty_input_lang').val(item[4]);

				CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.validateEndpointInput(dialog);
			}
		},
		{
			type: 'html',
			id: 'configWizardTabEndpointForm',
			html: '<div id="semwidgckConfigWizardEndpointForm"></div>'
		},
		{
			type: 'html',
			id: 'configWizardTabEndpointList',
			html: '<div id="semwidgckConfigWizardTabEndpointList"></div>'
		}

	];

	var resourceTab = {};
	resourceTab.id = 'resourceTab';
	resourceTab.label = 'Resources';

	resourceTab.elements = [
		{
			type: 'html',
			id: 'configWizardTabResourceAutoComplete',
			html: '<div id="semwidgckConfigWizardResourceAutoComplete"></div>'
		},
		{
			type: 'html',
			id: 'configWizardTabResourceForm',
			html: '<div id="semwidgckConfigWizardResourceForm"></div>'
		},
		{
			type: 'html',
			id: 'configWizardTabEndpointList',
			html: '<div id="semwidgckConfigWizardTabResourceList"></div>'
		},
		{
			type: 'html',
			id: 'dummy',
			html: ''
		}
	];
	
	dialog.contents.push(endpointTab);
	dialog.contents.push(resourceTab);

	dialog.buttons = [

		{
			id: 'previousButton',
			type: 'button',
			label: 'Previous',
			title: 'Previous Step',
			accessKey: 'P',
			disabled: false,
			onClick: function() {

				var confirmation = true;
				
				if (CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.highlightedButtons) {
					confirmation = confirm('You didn\'t add your new element. Are you sure that you want to proceed?');
				}

				if (confirmation) {

					CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.highlightedButtons = false;

					CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.applyDefaultEndpoint(dialog);

					var tabIndex = this.getDialog().definition.dialog._.currentTabIndex;
					var tabCount = this.getDialog().getPageCount();

					if (tabIndex - 1 >= 0) {
						var previousTab = this.getDialog().definition.dialog._.tabIdList[tabIndex - 1];
						this.getDialog().selectPage(previousTab);
					}
				}
			}
		},

		{
			id: 'nextButton',
			type: 'button',
			label: 'Next',
			title: 'Next Step',
			accessKey: 'N',
			disabled: false,
			onClick: function() {


				var confirmation = true;
				
				if (CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.highlightedButtons) {
					confirmation = confirm('You didn\'t add your new element. Are you sure that you want to proceed?');
				}

				if (confirmation) {

					CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.highlightedButtons = false;

					CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.applyDefaultEndpoint(dialog);

					var tabIndex = this.getDialog().definition.dialog._.currentTabIndex;
					var tabCount = this.getDialog().getPageCount();

					if (tabIndex + 1 < tabCount) {
						var nextTab = this.getDialog().definition.dialog._.tabIdList[tabIndex + 1];
						this.getDialog().selectPage(nextTab);
					}
				}

			}
		},

		CKEDITOR.dialog.okButton,
		CKEDITOR.dialog.cancelButton
	];

	dialog.onShow = function() {
		
		dialog.swEndpoints = CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.getEndpointsCopy(editor);
		dialog.swResources = CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.getResourcesCopy(editor);

		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardEndpointForm(dialog, editor);
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardEndpointList(dialog, editor);

		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardResourceAutoComplete(dialog, editor);
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardResourceForm(dialog, editor);
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardResourceList(dialog, editor);

		this.showPage('resourceTab');
	};

	dialog.onOk = function() {
		var confirmation = true;
				
		if (CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.unsavedResource) {
			confirmation = confirm('You didn\'t add your new element. Are you sure that you want to proceed?');
		}

		if (confirmation) {
			this.semwidgClosingType = 'ok';
			return true;
		} else {
			return false;
		}
		
	};

	dialog.onCancel = function() {
		var confirmation = true;
				
		if (CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.unsavedResource) {
			confirmation = confirm('You didn\'t add your new element. Are you sure that you want to proceed?');
		}

		if (confirmation) {
			this.semwidgClosingType = 'cancel';
			return true;
		} else {
			return false;
		}
	};

	dialog.onHide = function() {
		if (this.semwidgClosingType == 'ok') {

			var i = 0;

			// remove widgets
			var widgets = editor.widgets.instances;
			for (i in widgets) {
				var widget = widgets[i];
				var configType = widget.element.getAttribute('data-sw-config');
				if (configType == 'endpoint') {
					editor.widgets.del(widget);
				}else if (configType == 'resource') {
					editor.widgets.del(widget);
				}
			}

			var endpoints = dialog.swEndpoints;
			var endpoint;

			// set default endpoint
			CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.applyDefaultEndpoint(dialog);

			for (i = 0; i < endpoints.length; i++) {
				
				endpoint = endpoints[i];
				
				var html = '';
				html += '<div data-sw-config="endpoint" ';
				html += 'data-swp-id="' + (endpoint.getId() ? endpoint.getId() : '') + '" ';
				html += 'data-swp-url="' + (endpoint.getUrl() ? endpoint.getUrl() : '') + '" ';
				html += 'data-swp-default-graph-uri="' + (endpoint.getDefaultGraphUri() ? endpoint.getDefaultGraphUri() : '') + '" ';
				html += 'data-swp-default-lang="' + (endpoint.getDefaultLang() ? endpoint.getDefaultLang() : '') + '" ';
				html += 'data-swp-proxy="' + (endpoint.getProxy() ? endpoint.getProxy() : '') + '" ';
				html += 'data-swp-default="' + (endpoint.getDefault() ? endpoint.getDefault() : '') + '" ';
				html += '></div>';

				editor.editable().appendHtml(html);
			}

			var resources = dialog.swResources;
			var resource;

			for (i = 0; i < resources.length; i++) {
				
				resource = resources[i];

				var html = '';
				html += '<div data-sw-config="resource" ';
				html += 'data-swp-id="' + (resource.getId() ? resource.getId() : '') + '" ';
				html += 'data-swp-uri="' + (resource.getUri() ? resource.getUri() : '') + '" ';
				html += 'data-swp-endpoint="' + (resource.getEndpoint() ? resource.getEndpoint() : '') + '" ';
				html += '></div>';

				editor.editable().appendHtml(html);
			}
			
			editor.setMode('source');
			editor.setMode('wysiwyg');
		}
	};

	dialog.onLoad = function() {
		this.on('selectPage', function (e) {
			CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.applyDefaultEndpoint(dialog);

			var selectedTab = this._.currentTabIndex;
// MARKER
			if (selectedTab == 0) {
				// Check unsaved endpoint configs
				var endpointConfig = {};
				endpointConfig.id = $('#semwidgckConfigWizardProperty_input_endpointId').val();
				endpointConfig.url = $('#semwidgckConfigWizardProperty_input_endpointUrl').val();
				endpointConfig.defaultGraphUri = $('#semwidgckConfigWizardProperty_input_graphUri').val();
				endpointConfig.defaultLang = $('#semwidgckConfigWizardProperty_input_lang').val();

				var validEndpoint = CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.validateEndpointInput(dialog);
				// very simple check if url is set
				if (validEndpoint) {

					var endpoints = dialog.swEndpoints;

					var idIsUnique = true;
					for (var i = 0; i < endpoints.length; i++) {
						if (endpoints[i].getId() === endpointConfig.id) {
							idIsUnique = false;
							break;
						}
					}

					if (idIsUnique) {
						
						var confirmation = true;
						confirmation = confirm('You did not add your last endpoint configuration. Do you want to add it now?');

						if (confirmation) {
							CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.setupConfigWizardEndpointAdd(dialog, editor);
						}
					}

				}

			}
			
		});
	};

	return dialog;
});

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog = function() {

};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardEndpointForm = function(dialog, editor) {

	var configWizardEndpointForm = $('#semwidgckConfigWizardEndpointForm');
	var html = '';
	html += '<table style="width: 100%;">';

	// endpoint id
	html += '<tr>';
		html += '<td style="padding: 4px 0;">Endpoint ID:</td>';
		html += '<td style="padding: 4px 0px 4px 8px; width: 100%;"><input ';
		html += 'id="semwidgckConfigWizardProperty_input_endpointId" ';
		html += 'class="cke_dialog_ui_input_text" ';
	html += 'value="new">';
	html += '</td>';
	html += '</tr>';

	// endpoint url
	html += '<tr>';
		html += '<td style="padding: 4px 0;">Endpoint URL:</td>';
		html += '<td style="padding: 4px 0px 4px 8px; width: 100%;"><input ';
		html += 'id="semwidgckConfigWizardProperty_input_endpointUrl" ';
		html += 'class="cke_dialog_ui_input_text" ';
	html += 'value ="https://">';
	html += '</td>';
	html += '</tr>';

	// default graph uri
	html += '<tr>';
		html += '<td style="padding: 4px 0;">Default Graph URI:</td>';
		html += '<td style="padding: 4px 0px 4px 8px; width: 100%;"><input ';
		html += 'id="semwidgckConfigWizardProperty_input_graphUri" ';
		html += 'class="cke_dialog_ui_input_text" ';
	html += '></td>';
	html += '</tr>';

	// default language
	html += '<tr>';
		html += '<td style="padding: 4px 0;">Default Language:</td>';
		html += '<td style="padding: 4px 0px 4px 8px; width: 100%;"><input ';
		html += 'id="semwidgckConfigWizardProperty_input_lang" ';
		html += 'class="cke_dialog_ui_input_text" ';
	html += 'value ="en">';
	html += '</td>';
	html += '</tr>';

	html += '</table>';

	html += '<div style="text-align: right;">';
		html += '<a id="semwidgckConfigWizardAddEndpointButton" class="cke_dialog_ui_button" ';
				html += 'role="button" hidefocus="true" href="javascript:void(0)" ';
				html += 'style="margin: 4px 0;">';
			html += '<span class="cke_dialog_ui_button">Add Endpoint to List</span>';
		html += '</a>';
	html += '</div>';

	configWizardEndpointForm.html(html);

	var addEndpointButton = $('#semwidgckConfigWizardAddEndpointButton');
	addEndpointButton.click(function() {
		var inputValid = CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.validateEndpointInput(dialog);

		var confirmation = true;
		if (!inputValid) {
			confirmation = confirm('Probably the Endpoint URL and/or the Default Graph URI are malformed. Are you sure you want to add this endpoint?');
		}

		if (confirmation) {
			CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.setupConfigWizardEndpointAdd(dialog, editor);
		}
	});

	$('#semwidgckConfigWizardProperty_input_endpointId').on('keyup', function() {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.validateEndpointInput(dialog);
	});

	$('#semwidgckConfigWizardProperty_input_endpointUrl').on('keyup', function() {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.validateEndpointInput(dialog);
	});

	$('#semwidgckConfigWizardProperty_input_graphUri').on('keyup', function() {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.validateEndpointInput(dialog);
	});

	$('#semwidgckConfigWizardProperty_input_lang').on('keyup', function() {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.validateEndpointInput(dialog);
	});

	CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.validateEndpointInput(dialog);
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardEndpointList = function(dialog, editor) {

	var endpoints = dialog.swEndpoints;
	
	var configWizardEndpointList = $('#semwidgckConfigWizardTabEndpointList');
	var html = '';
	html += '<table style="width: 100%; border-bottom: 1px solid #BFBFBF; padding: 4px">';
	html += '<tr>';
		html += '<th style="width: 50px; padding: 4px">Default</th>';
		html += '<th style="width: 150px; padding: 4px">ID</th>';
		html += '<th style="padding: 4px;">URL</th>';
		html += '<th style="width: 20px; padding: 4px;"></th>';
	html += '</tr>';
	html += '</table>';

	html += '<table style="display: block; width: 100%; height: 132px; overflow: auto;">';
	var endpoint;
	for (var i = 0; i < endpoints.length; i++) {

		endpoint = endpoints[i];

		html += '<tr style="width: 100%; border-top: 1px solid #BFBFBF;">';
			html += '<td style="min-width: 50px; width: 50px; max-width: 50px; padding: 4px; text-align: center;">';
			html += '<input type="radio" name="defaultEndpoint" ';
			html += 'id="radio_defaultEndpoint_' + endpoint.getId() +'" ';
			
			if (endpoint.getDefault()) {
				html += 'checked="checked" ';
			}

			html += '">';
			html += '<td style="min-width: 150px; width: 150px; max-width: 150px; padding: 4px; overflow: hidden;">' + endpoint.getId() + '</td>';
			html += '<td style="width: 100%; padding: 4px; overflow: hidden;">' + endpoint.getUrl() + '</td>';
			html += '<td style="min-width: 20px; width: 20px; max-width: 20px; padding: 4px; overflow: hidden;"><a data-swql-removeEndpoint="' + endpoint.getId() + '" href="javascript:void(0);"><i class="fa fa-trash"></i></a></td>';
		html += '</tr>';
	}
	html += '</table>';

	configWizardEndpointList.html(html);

	var removeButtons = configWizardEndpointList.find('[data-swql-removeEndpoint]');
	var removeButton;
	for (var j = 0; j < removeButtons.length; j++) {
		removeButton = $(removeButtons[j]);

		removeButton.on('click', function(e) {
			var id = $(e.target).closest('a').attr('data-swql-removeEndpoint');
			CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.removeEndpoint(id, dialog, editor);
		});
	}

	CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.updateConfigWizardEndpointSelectList(dialog);
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.removeEndpoint = function(id, dialog, editor) {

	var index = -1;
	for (var i = 0; i < dialog.swEndpoints.length; i++) {
		if (id == dialog.swEndpoints[i].getId()) {
			index = i;
			break;
		}
	}

	if (index >= 0) {
		dialog.swEndpoints.splice(index, 1);
	}

	CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardEndpointList(dialog, editor);
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.setupConfigWizardEndpointAdd = function(dialog, editor) {

	var endpoint = new (editor.getSemwidgJS().Endpoint)();

	endpoint.setId($('#semwidgckConfigWizardProperty_input_endpointId').val());
	endpoint.setUrl($('#semwidgckConfigWizardProperty_input_endpointUrl').val());
	endpoint.setDefaultGraphUri($('#semwidgckConfigWizardProperty_input_graphUri').val());
	endpoint.setDefaultLang($('#semwidgckConfigWizardProperty_input_lang').val());

	var endpoints = dialog.swEndpoints;

	var idIsUnique = true;
	for (var i = 0; i < endpoints.length; i++) {
		if (endpoints[i].getId() === endpoint.getId()) {
			idIsUnique = false;
			break;
		}
	}

	if (idIsUnique) {
		endpoints.push(endpoint);
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardEndpointList(dialog, editor);
		
		$('#semwidgckConfigWizardProperty_input_endpointId').val('');
		$('#semwidgckConfigWizardProperty_input_endpointUrl').val('');
		$('#semwidgckConfigWizardProperty_input_graphUri').val('');
		$('#semwidgckConfigWizardProperty_input_lang').val('');

		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.validateEndpointInput(dialog);

	}else {
		alert('ID "' + endpoint.getId() + '" is already defined. IDs must be unique.');
	}
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog._defaultSparqlEndpoints;
CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.getDefaultSparqlEndpoints = function() {

	var dse = CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog._defaultSparqlEndpoints;

	if (!dse) {
		dse = [];

		dse.push(['<New Endpoint>', 'new', 'https://', '', 'en']);
		dse.push(['DBpedia', 'dbpedia', 'https://dbpedia.org/sparql', 'http://dbpedia.org', 'en']);
		dse.push(['DBpedia Interactive Systems Mirror', 'intsys', 'https://dbpedia.interactivesystems.info/sparql', 'http://dbpedia.org', 'en']);
		dse.push(['EAN Hotel Database', 'ean', 'https://dbpedia.interactivesystems.info/sparql', 'http://linkeddata.interactivesystems.info/ean', 'en']);
		dse.push(['Election Database', 'election', 'https://dbpedia.interactivesystems.info/sparql', 'http://linkeddata.interactivesystems.info/election', 'en']);
		dse.push(['Sensor Database', 'sensor_db', 'https://dbpedia.interactivesystems.info/sparql', 'http://linkeddata.interactivesystems.info/sensor', 'en']);
		// dse.push(['Linked Movie Database', 'lmdb', 'http://data.linkedmdb.org/sparql', '', 'en']);
		
	}

	return dse;
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.getEndpointsCopy = function(editor) {

	var endpointsRef = editor.getSemwidgJS().getConfig().getEndpoints();

	var endpointsCopy = [];

	var allowDefault = true;
	for (var i = 0; i < endpointsRef.length; i++) {
		var endpointCopy = new (editor.getSemwidgJS().Endpoint)();
		endpointCopy.setId(endpointsRef[i].getId());
		endpointCopy.setUrl(endpointsRef[i].getUrl());
		endpointCopy.setDefaultGraphUri(endpointsRef[i].getDefaultGraphUri());
		endpointCopy.setDefaultLang(endpointsRef[i].getDefaultLang());
		endpointCopy.setProxy(endpointsRef[i].getProxy());
		endpointCopy.setDefault(allowDefault ? endpointsRef[i].getDefault() : false);
		endpointCopy.setHttpMethod(endpointsRef[i].getHttpMethod());

		if (endpointCopy.getDefault()) {
			allowDefault = false;
		}

		endpointsCopy.push(endpointCopy);
	}

	return endpointsCopy;
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.applyDefaultEndpoint = function(dialog) {

	var endpoints = dialog.swEndpoints;
	var endpoint;

	// set default endpoint
	for (i = 0; i < endpoints.length; i++) {
		endpoint = endpoints[i];
		var elem = $('#radio_defaultEndpoint_' + endpoint.getId());
		endpoint.setDefault(elem.prop('checked'));
	}

	CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.updateConfigWizardEndpointSelectList(dialog);

};

// ############## Resources

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.unsavedResource = false;

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardResourceAutoComplete = function(dialog, editor) {

	var configWizardResourceAutoComplete = $('#semwidgckConfigWizardResourceAutoComplete');
	var html = '';
	html += '<table style="width: 100%;">';

	html += '<tr>';
		html += '<td style="padding: 4px 0;">Search:</td>';

		html += '<td style="width: 100%; padding: 4px 8px;">';
		html += '<input id="semwidgckConfigWizardProperty_input_autosuggest" ';
				html += 'class="cke_dialog_ui_input_text">';
		html += '</td>';

		html += '<td>';
		html += '<select id="semwidgckConfigWizardProperty_select_endpoint" ';
				html += 'class="cke_dialog_ui_input_select" style="width: 140px !important;"></select>';
		html += '</td>';
	html += '</tr>';
	html += '<tr>';
		html += '<td style="padding: 4px 0;"></td>';

		html += '<td style="width: 100%; padding: 4px 0px 4px 8px;" colspan="2">';
		html += '<select id="semwidgckConfigWizardProperty_select_resource" size="2" ';
				html += 'class="cke_dialog_ui_input_select" style="width: 100%; height: 90px;"></select>';
		html += '</td>';
	html += '</tr>';


	html += '</table>';

	configWizardResourceAutoComplete.html(html);

	CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.updateConfigWizardEndpointSelectList(dialog);

	var inputElem = $(configWizardResourceAutoComplete.find('#semwidgckConfigWizardProperty_input_autosuggest'));
	inputElem.on("keydown", function(e) {

		var keycode = e.keyCode;
		// arrow up and down
		if (keycode == 38 || keycode == 40) {
			e.preventDefault();

			// up
			if (keycode == 38) {

			}

			// down
			if (keycode == 40) {

			}

			return false;
		}

		// enter
		if (keycode == 13) {
			e.preventDefault();

			return false;
		}
	});

	inputElem.on('keyup', function(e) {

		var keycode = e.keyCode;
		// not enter, arrow up or down
		if (keycode != 13 && keycode != 38 && keycode != 40) {
			
			// search 
			var input = inputElem.val();
			var endpointId = $('#semwidgckConfigWizardProperty_select_endpoint').val();
			var endpoint;
			for (i = 0; i < dialog.swEndpoints.length; i++) {
				if (endpointId == dialog.swEndpoints[i].getId()) {
					endpoint = dialog.swEndpoints[i];
					break;
				}
			}
			CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.displayLoading();
			CKEDITOR.SemwidgJS.delay((function() {
				return CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.queryAutoSuggest(input, endpoint, CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.handleAutoSuggestResult, editor)
			}), 1000);
		}
	});

	var selectList = $(configWizardResourceAutoComplete.find('#semwidgckConfigWizardProperty_select_resource'));
	selectList.on('change', function(e) {
		var option = $(e.target).find('option:selected');
		var label = option.find('.semwidg-autocomplete-label').text();
		var uri = option.val();
		var endpoint = $('#semwidgckConfigWizardProperty_select_endpoint').find('option:selected');;

		// replace unallowed characters
		label = label.replace(/[^A-Za-z0-9\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\u1000-\uEFFF]/g, "_");
		label = label.toLowerCase();

		$('#semwidgckConfigWizardProperty_input_resourceId').val(label);
		$('#semwidgckConfigWizardProperty_input_resourceUri').val(uri);
		
		if (endpoint.text().lastIndexOf('<', 0) === 0) {
			$('#semwidgckConfigWizardProperty_input_resourceEndpoint').val();
		}else {
			$('#semwidgckConfigWizardProperty_input_resourceEndpoint').val(endpoint.val());
		}

		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.checkNewResource(dialog);
	});
	selectList.on('dblclick', function(e) {

		var option = $(e.target).closest('select').find('option:selected');
		var label = option.find('.semwidg-autocomplete-label').text();
		var uri = option.val();
		var endpoint = $('#semwidgckConfigWizardProperty_select_endpoint').find('option:selected').val();

		// replace unallowed characters
		label = label.replace(/[^A-Za-z0-9\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\u1000-\uEFFF]/g, "_");
		label = label.toLowerCase();

		var resource = new (editor.getSemwidgJS().Resource)();

		resource.setId(label);
		resource.setUri(uri);
		resource.setEndpoint(endpoint);

		var resources = dialog.swResources;

		var idIsUnique = true;
		for (var i = 0; i < resources.length; i++) {
			if (resources[i].getId() === resource.getId()) {
				idIsUnique = false;
				break;
			}
		}

		if (idIsUnique) {
			resources.push(resource);

			$('#semwidgckConfigWizardProperty_input_resourceId').val('');
			$('#semwidgckConfigWizardProperty_input_resourceUri').val('');
			$('#semwidgckConfigWizardProperty_input_resourceEndpoint').val('');

			CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardResourceList(dialog, editor);
			CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.checkNewResource(dialog);

			CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.unsavedResource = false;
		}else {
			alert('ID "' + resource.getId() + '" is already defined. IDs must be unique.');
		}
	});

};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardResourceForm = function(dialog, editor) {

	var configWizardResourceForm = $('#semwidgckConfigWizardResourceForm');
	var html = '';
	html += '<table style="width: 100%;">';

	// resource id
	html += '<tr>';
		html += '<td style="padding: 1px 0px 4px 0px;">Resource ID:</td>';
		html += '<td style="padding: 1px 0px 4px 8px; width: 100%;"><input ';
		html += 'id="semwidgckConfigWizardProperty_input_resourceId" ';
		html += 'class="cke_dialog_ui_input_text" ';
	html += 'value="new">';
	html += '</td>';
	html += '</tr>';

	// resource uri
	html += '<tr>';
		html += '<td style="padding: 4px 0;">Resource URI:</td>';
		html += '<td style="padding: 4px 0px 4px 8px; width: 100%;"><input ';
		html += 'id="semwidgckConfigWizardProperty_input_resourceUri" ';
		html += 'class="cke_dialog_ui_input_text" ';
	html += 'value ="http://">';
	html += '</td>';
	html += '</tr>';

	// endpoint
	html += '<tr>';
		html += '<td style="padding: 4px 0;">Endpoint:</td>';
		html += '<td style="padding: 4px 0px 4px 8px; width: 100%;"><input ';
		html += 'id="semwidgckConfigWizardProperty_input_resourceEndpoint" ';
		html += 'class="cke_dialog_ui_input_text" ';
	html += '></td>';
	html += '</tr>';

	html += '</table>';

	html += '<div style="text-align: right;">';
		html += '<a id="semwidgckConfigWizardAddResourceButton" class="cke_dialog_ui_button" ';
				html += 'role="button" hidefocus="true" href="javascript:void(0)" ';
				html += 'style="margin: 4px 0;">';
			html += '<span class="cke_dialog_ui_button">Add Resource</span>';
		html += '</a>';
	html += '</div>';

	configWizardResourceForm.html(html);

	var addResourceButton = $('#semwidgckConfigWizardAddResourceButton');
	addResourceButton.click(function() {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.setupConfigWizardResourceAdd(dialog, editor);
	});

	$('#semwidgckConfigWizardProperty_input_resourceId').on('keyup', function() {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.checkNewResource(dialog);
	});

	$('#semwidgckConfigWizardProperty_input_resourceUri').on('keyup', function() {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.checkNewResource(dialog);
	});

	$('#semwidgckConfigWizardProperty_input_resourceEndpoint').on('keyup', function() {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.checkNewResource(dialog);
	});

	CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.checkNewResource(dialog);
};


CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.checkNewResource = function(dialog) {

	var idIsValid = true;
	var resources = dialog.swResources;
	var resourceId = $('#semwidgckConfigWizardProperty_input_resourceId').val();

	if (resourceId != '') {
		for (var i = 0; i < resources.length; i++) {
			if (resources[i].getId() === resourceId) {
				idIsValid = false;
				break;
			}
		}
	}else {
		idIsValid = false;
	}

	if (idIsValid){
		$('#semwidgckConfigWizardProperty_input_resourceId').attr('data-swql-valid', 'true');
	}else {
		$('#semwidgckConfigWizardProperty_input_resourceId').attr('data-swql-valid', 'false');
	}


	var uri = $('#semwidgckConfigWizardProperty_input_resourceUri').val();
	var uriIsValid = CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.urlRegEx.test(uri);
	if (uriIsValid) {
		$('#semwidgckConfigWizardProperty_input_resourceUri').attr('data-swql-valid', 'true');
	}else {
		$('#semwidgckConfigWizardProperty_input_resourceUri').attr('data-swql-valid', 'false');
	}

	var endpointIsValid = false;
	var endpoints = dialog.swEndpoints;
	var endpointId = $('#semwidgckConfigWizardProperty_input_resourceEndpoint').val();

	if (endpointId != '') {
		for (var i = 0; i < endpoints.length; i++) {
			if (endpoints[i].getId() === endpointId) {
				endpointIsValid = true;
				break;
			}
		}
	}else {
		endpointIsValid = true;
	}
	if (endpointIsValid){
		$('#semwidgckConfigWizardProperty_input_resourceEndpoint').attr('data-swql-valid', 'true');
	}else {
		$('#semwidgckConfigWizardProperty_input_resourceEndpoint').attr('data-swql-valid', 'false');
	}
	
	var isValid = idIsValid && uriIsValid && endpointIsValid;
	if (isValid){
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.highlightAddButton('semwidgckConfigWizardAddResourceButton', ['semwidg-button-ok'], ['semwidg-button-warning']);
		// CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.unsavedResource = true;
	}else {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.highlightAddButton('semwidgckConfigWizardAddResourceButton', ['semwidg-button-warning'], ['semwidg-button-ok']);
	}

	if (resourceId && resourceId.length > 0 && resourceId !== 'new'
			&& uri && uri.length > 0 && uri !== 'http://') {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.unsavedResource = true;
	}
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.highlightAddButton = function(id, addStylesArray, removeStylesArray) {

	if (!addStylesArray || !Array.isArray(addStylesArray)) {
		addStylesArray = [];
	}

	if (!removeStylesArray || !Array.isArray(removeStylesArray)) {
		removeStylesArray = [];
	}

	var addButton = $('#' + id);
	addButton.removeClass(removeStylesArray.join(' ')).addClass(addStylesArray.join(' '));
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardResourceList = function(dialog, editor) {

	var resources = dialog.swResources;
	
	var configWizardResourceList = $('#semwidgckConfigWizardTabResourceList');
	var html = '';
	html += '<table style="width: 100%; border-bottom: 1px solid #BFBFBF;">';
	html += '<tr>';
		html += '<th style="width: 150px; padding: 4px">ID</th>';
		html += '<th style="width: 400px; padding: 4px">Resource URI</th>';
		html += '<th style="padding: 4px;">Endpoint</th>';
	html += '</tr>';
	html += '</table>';

	html += '<table style="display: block; width: 100%; height: 72px; overflow: auto;">';
	var resource;
	for (var i = 0; i < resources.length; i++) {

		resource = resources[i];

		html += '<tr style="width: 100%; border-top: 1px solid #BFBFBF;">';
			html += '<td style="min-width: 150px; width: 150px; max-width: 150px; padding: 4px; overflow: hidden;">' + resource.getId() + '</td>';
			html += '<td style="min-width: 400px; width: 400px; max-width: 400px; padding: 4px; overflow: hidden;">' + resource.getUri() + '</td>';
			html += '<td style="width: 100%; padding: 4px; overflow: hidden;">' + resource.getEndpoint() + '</td>';
			html += '<td style="min-width: 20px; width: 20px; max-width: 20px; padding: 4px; overflow: hidden;"><a data-swql-removeResource="' + resource.getId() + '" href="javascript:void(0);"><i class="fa fa-trash"></i></a></td>';
		html += '</tr>';
	}
	html += '</table>';

	configWizardResourceList.html(html);

	var removeButtons = configWizardResourceList.find('[data-swql-removeResource]');
	var removeButton;
	for (var j = 0; j < removeButtons.length; j++) {
		removeButton = $(removeButtons[j]);

		removeButton.on('click', function(e) {
			var id = $(e.target).closest('a').attr('data-swql-removeResource');
			CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.removeResource(id, dialog, editor);
		});
	}

};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.removeResource = function(id, dialog, editor) {

	var index = -1;
	for (var i = 0; i < dialog.swResources.length; i++) {
		if (id == dialog.swResources[i].getId()) {
			index = i;
			break;
		}
	}
	
	if (index >= 0) {
		dialog.swResources.splice(index, 1);
	}

	CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardResourceList(dialog, editor);
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.updateConfigWizardEndpointSelectList = function(dialog) {

	var endpoints = dialog.swEndpoints;
	var endpointSelect = $('#semwidgckConfigWizardProperty_select_endpoint');
	
	endpointSelect.find('option').remove();

	var i = 0;
	// get Default Endpoint
	for (i = 0; i < endpoints.length; i++) {
		if (endpoints[i].getDefault()) {
			var option = $('<option></option>').attr('value', endpoints[i].getId()).text('<Default Endpoint>');
			endpointSelect.append(option);
			break;
		}
	}

	for (i = 0; i < endpoints.length; i++) {
		var option = $('<option></option>').attr('value', endpoints[i].getId()).text(endpoints[i].getId());
		endpointSelect.append(option);
	}

};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.setupConfigWizardResourceAdd = function(dialog, editor) {

	var resource = new (editor.getSemwidgJS().Resource)();

	resource.setId($('#semwidgckConfigWizardProperty_input_resourceId').val());
	resource.setUri($('#semwidgckConfigWizardProperty_input_resourceUri').val());
	resource.setEndpoint($('#semwidgckConfigWizardProperty_input_resourceEndpoint').val());

	var resources = dialog.swResources;

	var idIsUnique = true;
	for (var i = 0; i < resources.length; i++) {
		if (resources[i].getId() === resource.getId()) {
			idIsUnique = false;
			break;
		}
	}

	if (idIsUnique) {
		resources.push(resource);

		$('#semwidgckConfigWizardProperty_input_resourceId').val('');
		$('#semwidgckConfigWizardProperty_input_resourceUri').val('');
		$('#semwidgckConfigWizardProperty_input_resourceEndpoint').val('');

		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.buildConfigWizardResourceList(dialog, editor);
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.checkNewResource(dialog);

		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.unsavedResource = false;
		
	}else {
		alert('ID "' + resource.getId() + '" is already defined. IDs must be unique.');
	}
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.getResourcesCopy = function(editor) {

	var resourcesRef = editor.getSemwidgJS().getConfig().getResources();

	var resourcesCopy = [];

	for (var i = 0; i < resourcesRef.length; i++) {
		var resourceCopy = new (editor.getSemwidgJS().Resource)();
		resourceCopy.setId(resourcesRef[i].getId());
		resourceCopy.setUri(resourcesRef[i].getUri());
		resourceCopy.setEndpoint(resourcesRef[i].getEndpoint());
		
		resourcesCopy.push(resourceCopy);
	}

	return resourcesCopy;
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.queryAutoSuggest = function(value, endpoint, callback, editor) {
	
	var query = CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.createAutoSuggestQuery(value, endpoint.getDefaultLang());

	if (!query) {
		return;
	}

	var SemwidgJS = editor.getSemwidgJS();

	var url;
	if (endpoint) {
		var proxy = endpoint.getProxy();

		if (SemwidgJS.Util.String.isEmpty(proxy)) {
			proxy = 'none';
		}

		if (proxy != 'none') {

			if (proxy == 'default') {
				url = SemwidgJS.Util.getDefaultProxy() + '?url=' + endpoint.getUrl();
			}else {
				url = SemwidgJS.getConfig().getProxy(proxy).getUrl() + '?url=' + endpoint.getUrl();
			}
			
		}else {
			url = endpoint.getUrl();
		}

	}else {
		console.error('No Endpoint found for query!')
		return;
	}

	var method = ('' + endpoint.getHttpMethod()).toUpperCase();

	if (method == 'GET') {
		if (proxy == 'none') {
			url += '?';
		}
		url += '&format=JSON';
		url += '&output=JSON';
		if (!SemwidgJS.Util.String.isEmpty(endpoint.getDefaultGraphUri())){
			url += '&default-graph-uri=' + encodeURIComponent(endpoint.getDefaultGraphUri());
		}
		url += '&query=' + encodeURIComponent(query);
	}

	var cachedResult = SemwidgJS.SparqlCache.getResult(
			query, 
			endpoint.getUrl(), 
			endpoint.getDefaultGraphUri(), 
			'JSON'
		);

	if (cachedResult && cachedResult.getStatus() > 0) {
		var dummyResponse = new Object();
		
		dummyResponse._swSparqlQuery = cachedResult.getSparqlQuery();
		dummyResponse._swEndpointUrl = cachedResult.getEndpointUrl();
		dummyResponse._swDefaultGraphUri = cachedResult.getDefaultGraphUri();
		dummyResponse._swFormat = cachedResult.getFormat();

		dummyResponse._swEditor = editor;
		dummyResponse._swCallback = callback;
		
		dummyResponse.responseText = cachedResult.getResult();
		dummyResponse.status = 200;

		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.handleRequestResult(dummyResponse);
	}else {

		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.doRequest(
			CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.getCounter(), 
			query, 
			url, 
			endpoint.getDefaultGraphUri(), 
			'JSON',
			editor,
			callback,
			method
		);
	}
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.createAutoSuggestQuery = function(value, lang) {

	if (!value || value == '') {
		return;
	}

	value = value.replace(/([.*+?^=!:$"'#<>{}()|\[\]\/\\])/g, ' ');

	var query = '';

	query += 'SELECT ?s ?l ';
	query += 'WHERE { ';
	query += '?s <http://www.w3.org/2000/01/rdf-schema#label> ?l . ';
	
	// query += 'FILTER regex(?l, "' + value + '", "i") . ';

	var prepareValue = function(value) {
		var valArr = value.split(' ');
		var v = '';
		for (var i = 0; i < valArr.length; i++) {
			if (valArr[i].length > 0) {

				if (i > 0) {
					v += '\' AND \'';
				}

				if (valArr[i].length < 4) {
					v += valArr[i];
				}else {
					v += valArr[i] + '*';
				}

			}
		}
		return v;
	}

	value = prepareValue(value);

	query += '?l <bif:contains> "\'' + value + '\'" . ';
	
	query += 'OPTIONAL { ';
		query += '?res a <http://www.w3.org/2004/02/skos/core#Concept> . ';
		query += 'FILTER(?res = ?s) ';
		query += '} ';
	query += 'FILTER(!bound(?res)) . ';

	if (lang) {
		query += 'FILTER(lang(?l) = "" || langMatches(lang(?l), "' + lang  + '")) . ';
	}

	query += '} ';
	query += 'LIMIT 1000 ';

	return query;
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog._counter = 1;
CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.getCounter = function() {
	return CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog._counter++;
}

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog._lastSparqlRequestNumber = 0;
CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.doRequest = function(requestNumber, sparqlQuery, endpointUrl, defaultGraphUri, format, editor, callback, httpMethod) {

	var SemwidgJS = editor.getSemwidgJS();

	if (requestNumber > CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog._lastSparqlRequestNumber) {

		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog._lastSparqlRequestNumber = requestNumber;

		var request;

		if (httpMethod == 'GET') {
			request = $.get(endpointUrl);
		}else {
			request = $.post(
				endpointUrl,
				{
					'format': format,
					'output': format,
					'default-graph-uri': defaultGraphUri,
					'query': sparqlQuery
				}
			);
		}
		

		SemwidgJS.SparqlCache.insertResult(
			sparqlQuery,
			endpointUrl,
			defaultGraphUri,
			format,
			SemwidgJS.SparqlCache.ResultObject.STATUS_REQUESTING);

		request._swSparqlQuery = sparqlQuery;
		request._swEndpointUrl = endpointUrl;
		request._swDefaultGraphUri = defaultGraphUri;
		request._swFormat = format;

		request._swEditor = editor;
		request._swCallback = callback;

		request.complete(CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.handleRequestResult);

	}
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.handleRequestResult = function(data) {
	
	if (!data || data.status != 200) {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.displayError(data);
	}

	var SemwidgJS = data._swEditor.getSemwidgJS();

	SemwidgJS.SparqlCache.insertResult(
			data._swSparqlQuery,
			data._swEndpointUrl,
			data._swDefaultGraphUri,
			data._swFormat,
			SemwidgJS.SparqlCache.ResultObject.STATUS_FOUND,
			data.responseText);

	if (data._swCallback) {
		data._swCallback(data);
	}
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.handleAutoSuggestResult = function(data) {

	var resultList = $('#semwidgckConfigWizardProperty_select_resource');
	var searchTerm = $('#semwidgckConfigWizardProperty_input_autosuggest').val();

	var result = CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.sparqlJsonToObject(data.responseText);
	result = CKEDITOR.SemwidgJS.AutoSuggest.filterResult(result, searchTerm);

	resultList.find('option').remove();

	if (result.length > 0) {
		for (var i = 0; i < result.length; i++) {
			var option = $('<option></option>').attr('value', CKEDITOR.SemwidgJS.sanitizeHtml(result[i].getValue())).html(CKEDITOR.SemwidgJS.sanitizeHtml(result[i].toHtml()));
			resultList.append(option);
		}
	}else {
		var option = $('<option></option>').attr('value', '').html('Sorry, there are no matching search results.');
		resultList.append(option);
	}
	

};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.displayError = function(data) {

	var resultList = $('#semwidgckConfigWizardProperty_select_resource');

	resultList.find('option').remove();

	var option = $('<option></option>').attr('value', '').html('An error occured: Sorry, but I don\'t know what went wrong.');
	resultList.append(option);

};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.displayLoading = function() {

	var resultList = $('#semwidgckConfigWizardProperty_select_resource');

	resultList.find('option').remove();

	var option = $('<option></option>').attr('value', '').html('Loading...');
	resultList.append(option);

};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.sparqlJsonToObject = function(jsonResult) {

	if (!jsonResult || jsonResult == '') {
		return;
	}

	jsonResult = $.parseJSON(jsonResult);

	var result;

	if (jsonResult) {

		result = new Array();

		var bindings = jsonResult.results.bindings;

		for (var i = 0; i < bindings.length; i++) {

			var binding = bindings[i];

			var elem = new CKEDITOR.SemwidgJS.AutoSuggest.ResultElement();

			if (binding.hasOwnProperty('s')) {
				var s = binding.s.value;
				try {
					s = decodeURIComponent(s);
				}catch (e) {}
				elem.setValue(s);
			}

			if (binding.hasOwnProperty('l')) {
				var l = binding.l.value;
				try {
					l = decodeURIComponent(l);
				}catch (e) {}
				elem.setLabel(l);
			}

			result.push(elem);
		}

	}

	return result;
};

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.urlRegEx = 
		/^([a-z][a-z0-9+.-]*):(?:\/\/((?:(?=((?:[a-z0-9-._~!$&'()*+,;=:]|%[0-9A-F]{2})*))(\3)@)?(?=(\[[0-9A-F:.]{2,}\]|(?:[a-z0-9-._~!$&'()*+,;=]|%[0-9A-F]{2})*))\5(?::(?=(\d*))\6)?)(\/(?=((?:[a-z0-9-._~!$&'()*+,;=:@\/]|%[0-9A-F]{2})*))\8)?|(\/?(?!\/)(?=((?:[a-z0-9-._~!$&'()*+,;=:@\/]|%[0-9A-F]{2})*))\10)?)(?:\?(?=((?:[a-z0-9-._~!$&'()*+,;=:@\/?]|%[0-9A-F]{2})*))\11)?(?:#(?=((?:[a-z0-9-._~!$&'()*+,;=:@\/?]|%[0-9A-F]{2})*))\12)?$/i;

CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.validateEndpointInput = function(dialog) {
	
	var id = $('#semwidgckConfigWizardProperty_input_endpointId').val();
	var url = $('#semwidgckConfigWizardProperty_input_endpointUrl').val();
	var graph = $('#semwidgckConfigWizardProperty_input_graphUri').val();
	
	var idIsValid = true;
	var endpoints = dialog.swEndpoints;
	if (id != '') {
		for (var i = 0; i < endpoints.length; i++) {
			if (endpoints[i].getId() === id) {
				idIsValid = false;
				break;
			}
		}
	}else {
		idIsValid = false;
	}

	if (idIsValid){
		$('#semwidgckConfigWizardProperty_input_endpointId').attr('data-swql-valid', 'true');
	}else {
		$('#semwidgckConfigWizardProperty_input_endpointId').attr('data-swql-valid', 'false');
	}


	var urlIsValid = CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.urlRegEx.test(url);
	urlIsValid = urlIsValid && (url !== 'https://');

	if (urlIsValid){
		$('#semwidgckConfigWizardProperty_input_endpointUrl').attr('data-swql-valid', 'true');
	}else {
		$('#semwidgckConfigWizardProperty_input_endpointUrl').attr('data-swql-valid', 'false');
	}

	var graphIsValid = false;
	if (graph != '') {
		graphIsValid = CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.urlRegEx.test(graph);
	}else {
		graphIsValid = true;
	}

	if (graphIsValid){
		$('#semwidgckConfigWizardProperty_input_graphUri').attr('data-swql-valid', 'true');
	}else {
		$('#semwidgckConfigWizardProperty_input_graphUri').attr('data-swql-valid', 'false');
	}


	var isValid = idIsValid && urlIsValid && graphIsValid;

	if (isValid) {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.highlightAddButton('semwidgckConfigWizardAddEndpointButton', ['semwidg-button-ok'], ['semwidg-button-warning']);
	}else {
		CKEDITOR.SemwidgJS.SemwidgConfigWizardDialog.highlightAddButton('semwidgckConfigWizardAddEndpointButton', ['semwidg-button-warning'], ['semwidg-button-ok']);
	}

	return isValid;
};