CKEDITOR.dialog.add( 'semwidgWidgetDialog', function (editor) {

	var dialog = {};

	dialog.title = 'SemwidgJS Widget';
	dialog.minWidth = 800;
	dialog.minHeight = 400;

	dialog.contents = [];

	var widgetTab = {};
	widgetTab.id = 'widgetProperties';
	widgetTab.label = 'Widget Properties';

	widgetTab.elements = [
		{
			type: 'html',
			id: 'widgetTabHtmlInfo',
			html: '<div><h1 id="semwidgckWidgetTabTitle">Enter Widget Properties:</h1></div>'
		},
		{
			type: 'html',
			id: 'widgetTabHtmlForm',
			html: '<div id="semwidgckWidgetTab" style="height: 360px; overflow: auto; position: relative;"></div>',
			setup: function(widget) {
				CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setupWidgetTab(widget, editor);
			},
			commit: function(widget) {
				CKEDITOR.SemwidgJS.SemwidgWidgetDialog.commitWidgetTab(widget, editor);
			}
		}
	];

	dialog.contents.push(widgetTab);

	var basicTab = {};
	basicTab.id = 'basicProperties';
	basicTab.label = 'Basic Properties';

	basicTab.elements = [
		{
			type: 'html',
			id: 'basicTabHtmlInfo',
			html: '<div><h1 id="semwidgckBasicTabTitle">Enter Widget Properties:</h1></div>'
		},
		{
			type: 'html',
			id: 'basicTabHtmlForm',
			html: '<div id="semwidgckBasicTab" style="height: 360px; overflow: auto; position: relative;"></div>',
			setup: function(widget) {
				CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setupBasicTab(widget, editor);
			},
			commit: function(widget) {
				CKEDITOR.SemwidgJS.SemwidgWidgetDialog.commitBasicTab(widget, editor);
			}
			
		}
	];

	dialog.contents.push(basicTab);

	dialog.onShow = function() {

		if (CKEDITOR.SemwidgJS.data.widgetType && CKEDITOR.SemwidgJS.data.widgetType.lastIndexOf('widget_', 0) === 0) {

			editor.semwidgWidgetType = CKEDITOR.SemwidgJS.data.widgetType.substr(7);
			CKEDITOR.SemwidgJS.SemwidgWidgetDialog.buildWidgetDialog(this, editor);

			CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setDefaultWidgetData(editor.semwidgWidgetType);
		}
	};

	dialog.onOk = function() {
		this.semwidgClosingType = 'ok';
	};

	dialog.onCancel = function() {
		this.semwidgClosingType = 'cancel';
	};

	dialog.onHide = function() {
		if (this.semwidgClosingType == 'ok') {
			editor.setMode('source');
			editor.setMode('wysiwyg');
		}
	}

	return dialog;
});

CKEDITOR.SemwidgJS.SemwidgWidgetDialog = function() {

};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.tempModel = {};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.buildWidgetDialog = function(dialog, editor) {

	CKEDITOR.SemwidgJS.SemwidgWidgetDialog.tempModel = {};

	var type = editor.semwidgWidgetType;

	var definition = dialog.definition;
	var widgetTab = definition.getContents('widgetProperties');

	// build Widget Tab
	var widgetTypes = editor.getSemwidgJS().WidgetProvider.getWidgetTypes();
	
	var typeHtml = '<select id="semwidgckWidgetTabTypeSelect" style="border: 1px solid #bfbfbf">';
	for (var widgetType in widgetTypes) {
		if (widgetType && widgetType != 'SwWidget') {
			if (widgetType == type) {
				typeHtml += '<option value="' + widgetType + '" selected>' + widgetType + '</option>';
			}else {
				typeHtml += '<option value="' + widgetType + '">' + widgetType + '</option>';
			}
		}
	}
	typeHtml += '</select>';
	$('#semwidgckWidgetTabTitle').html('Enter Widget Properties for ' + typeHtml);

	$('#semwidgckWidgetTabTypeSelect').on('change', function() {
		editor.semwidgWidgetType = this.value;

		var inputElements = $('#semwidgckWidgetTab').find('[id^=semwidgckWidgetProperty_input]');
		var inputElement;
		for (var i = 0; i < inputElements.length; i++) {
			inputElement = $(inputElements[i]);

			var value = inputElement.text();

			CKEDITOR.SemwidgJS.SemwidgWidgetDialog.tempModel[inputElement.attr('id')] = value;
		}

		buildWidgetTab(editor.semwidgWidgetType);
		CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setupWidgetTab(null, editor);
		CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setDefaultWidgetData(editor.semwidgWidgetType);
	});

	var widgetTab = $('#semwidgckWidgetTab');
	var widgetProperties;

	var additionalProperties = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAdditionalProperties(editor);

	var buildWidgetTab = function(type) {
		widgetProperties = CKEDITOR.SemwidgJS.getWidgetProperties(type, editor);
		var widgetTabHtml = '';
		widgetTabHtml += '<table style="width: 100%;">';
		for (var i = 0; i < widgetProperties.length; i++) {
			var property = CKEDITOR.SemwidgJS.getProperty(type, widgetProperties[i], editor);
			widgetTabHtml += '<tr title="' + property[2] + '" >';
			widgetTabHtml += '<td style="padding: 4px 0;">';

			if (('' + property[1]).toLowerCase() == 'query') {
				widgetTabHtml += '<a href="javascript:void(0)" ' + 
						'onclick="CKEDITOR.SemwidgJS.SemwidgWidgetDialog.toggleAdditionalProperties(this, ' +
								'\'semwidgckWidgetProperty_table_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]) + '\')"' + 
					'><i class="fa fa-plus-square-o"></i></a>';
			}else {
				widgetTabHtml += '<i class="fa" style="width: 9px";>&nbsp;</i>';
			}

			widgetTabHtml += '<span style="padding-left: 6px;">' + property[0] + ':</span>';

			widgetTabHtml += '</td>';
			widgetTabHtml += '<td style="padding: 4px 0px 4px 8px; width: 100%;">';
				widgetTabHtml += '<div ';
				widgetTabHtml += 'id="semwidgckWidgetProperty_input_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]) + '" ';
				widgetTabHtml += 'class="semwidg-autosuggest-input" ';
				widgetTabHtml += 'contentEditable="true">&nbsp;</div>';

			if (('' + property[1]).toLowerCase() == 'query') {
				widgetTabHtml += '<table style="width: 100%; display: none;" ' + 
					'id="semwidgckWidgetProperty_table_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]) + '">';

				for (var j = 0; j < additionalProperties.length; j++) {
					widgetTabHtml += '<tr title="' + additionalProperties[j][2] + '" >';
					widgetTabHtml += '<td style="padding: 4px 0;">-' + additionalProperties[j][0] + ':</td>';
					widgetTabHtml += '<td style="padding: 4px 0px 4px 8px; width: 100%;">';
						widgetTabHtml += '<div ';
						widgetTabHtml += 'id="semwidgckWidgetProperty_input_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]) + '_' + additionalProperties[j][0] + '" ';
						widgetTabHtml += 'class="semwidg-autosuggest-input" ';
						widgetTabHtml += 'contentEditable="true">&nbsp;</div>';
					widgetTabHtml += '</td>';
					widgetTabHtml += '</tr>';
				}

				widgetTabHtml += '</table>';
			}

			widgetTabHtml += '</td>';
			widgetTabHtml += '</tr>';
		}
		widgetTabHtml += '</table>';

		widgetTab.html(widgetTabHtml);

		for (var i = 0; i < widgetProperties.length; i++) {
			var property = CKEDITOR.SemwidgJS.getProperty(type, widgetProperties[i], editor);
			if (('' + property[1]).toLowerCase() == 'query') {
				
				var id = 'semwidgckWidgetProperty_input_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]);
				var propertyAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id, false);

				propertyAutosuggest.setSelectFunction(CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAutosuggestSelectFunction(propertyAutosuggest, editor));
				propertyAutosuggest.setRequestFunction(CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAutosuggestRequestFunction(propertyAutosuggest, editor), 0);

				propertyAutosuggest.element.on('paste', function(e) {
					CKEDITOR.SemwidgJS.SemwidgWidgetDialog.updateQueryInput($(e.currentTarget), editor);
				});

				propertyAutosuggest.element.on('keyup', function(e) {
					CKEDITOR.SemwidgJS.SemwidgWidgetDialog.updateQueryInput($(e.currentTarget), editor);
				});

				propertyAutosuggest.element.on('keydown', function(e) {
					if (e.keyCode == 32) {
						e.preventDefault();
						document.execCommand('insertHtml', false, '&nbsp;');
					}
				});

				var id2;

				id2 = id + '_Order';
				propertyAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id2, false);

				propertyAutosuggest.setSelectFunction(function(value) {
					this.element.text(value);
					this.element.focus();
				});

				propertyAutosuggest.setRequestFunction(function(value) {

					var recommendations = ['last asc', 'last desc', 'last - 1 asc', 'first + 1', 'first'];
					var resultContains = [];
					var resultContainsNot = [];

					if (recommendations) {
						for (var i = 0; i < recommendations.length; i++) {
							if (recommendations[i].indexOf(value) > -1) {
								resultContains.push(recommendations[i]);
							}else {
								resultContainsNot.push(recommendations[i]);
							}
						}
					};

					var simpleStringArraySort = function(a, b) {
						var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a, value);
						var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b, value);

						return levA - levB;
					};

					resultContains.sort(simpleStringArraySort);
					resultContainsNot.sort(simpleStringArraySort);

					var result = resultContains.concat(resultContainsNot);

					for (var i = 0; i < result.length; i++) {
						result[i] = ['<span style="font-family: monospace;">' + result[i] + '</span>', result[i]];
					}

					this.setResult(result);
					
				}, 0);	

				id2 = id + '_Endpoint';
				propertyAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id2, false);

				propertyAutosuggest.setSelectFunction(function(value) {
					this.element.text(value);
					this.element.focus();
				});

				propertyAutosuggest.setRequestFunction(function(value) {

					var endpoints = editor.getSemwidgJS().getConfig().getEndpoints();
					var resultContains = [];
					var resultContainsNot = [];

					if (endpoints) {
						for (var i = 0; i < endpoints.length; i++) {
							if (endpoints[i].getId().indexOf(value) > -1) {
								resultContains.push(endpoints[i].getId());
							}else {
								resultContainsNot.push(endpoints[i].getId());
							}
						}
					};

					var simpleStringArraySort = function(a, b) {
						var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a, value);
						var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b, value);

						return levA - levB;
					};

					resultContains.sort(simpleStringArraySort);
					resultContainsNot.sort(simpleStringArraySort);

					var result = resultContains.concat(resultContainsNot);

					for (var i = 0; i < result.length; i++) {
						result[i] = ['<span style="font-family: monospace;">' + result[i] + '</span>', result[i]];
					}

					this.setResult(result);
					
				}, 0);												


			}else if (('' + property[1]).toLowerCase() == 'selector') {

				var id = 'semwidgckWidgetProperty_input_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]);
				var selectorAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id, false);

				selectorAutosuggest.setSelectFunction(function(value) {
					this.element.text(value);
					this.element.focus();
				});
				selectorAutosuggest.setRequestFunction(function(value) {

					var cursorPosition = CKEDITOR.SemwidgJS.getCursorPosition(this.element[0]);
					var subquery = value.substr(0, cursorPosition);

					var result = []
					
					var s = '';
					var dots = 0;
					var lastChar;
					for (var i = subquery.length - 1; i >= 0; i--) {

						s = subquery.substr(0, i);
						 lastChar = subquery.charAt(i);

						if (lastChar == '.') {
							dots++;
						}

					}

					if (dots == 0) {
						var elements = editor.getSemwidgJS().Util.getAllSemwidgElements();
						for (var i = 0;  i < elements.length; i++) {
							result.push(['<span style="font-family: monospace;">' + elements[i].getId() + '</span> (' + elements[i]._getType() + ')', elements[i].getId() + '.']);
						}
					}else if (dots == 1) {
						var elementId = subquery.substr(0, subquery.indexOf('.'));
						var element = editor.getSemwidgJS().Util.getSemwidgElementById(elementId);

						if (element) {

							var type = element._getType();
							var parentType;

							if (!element._node) {
								parentType = 'semwidgConfig';
							}else {
								parentType = element._node.getAttribute('data-widget');
							}

							var attributes;
							if (parentType == 'semwidgConfig') {
								attributes = CKEDITOR.SemwidgJS.getConfigProperties(type.substr(2));
							}else if (parentType == 'semwidgWidget') {
								var properties = element._getProperties();
								attributes = [];
								for (var prop in properties) {
									attributes.push(prop);
								}
							}else if (parentType == 'semwidgTemplate') {
								attributes = ['id', 'parameters'];
							}else if (parentType == 'semwidgTemplateInstance') {
								var parameters = element.getParameters();
								attributes = [];
								for (var param in parameters) {
									attributes.push(param);
								}
							}

							for (var i = 0; i < attributes.length; i++) {
								result.push(['<span style="font-family: monospace;">' + element.getId() + '.' + attributes[i] + '</span> (' + element._getType() + ')', element.getId() + '.' + attributes[i]]);
							}

						}else {
							result.push(['<span>Error: Element "' + elementId + '"" was not found</span>', '']);
						}

					}else {
						result.push(['<span>Error: Selectors must look like "ElementId.PropertyName"</span>', '']);
					}
					this.setResult(result);

				}, 0)
			}
		}
	}
	
	buildWidgetTab(type);

	// build Basic Tab
	$('#semwidgckBasicTabTitle').text('Enter Basic Properties for ' + type + ':');

	var basicTab = $('#semwidgckBasicTab');
	var basicProperties = CKEDITOR.SemwidgJS.getBasicWidgetProperties(editor);

	var basicTabHtml = '';
	basicTabHtml += '<table style="width: 100%;">';

	// HTML type
	basicTabHtml += '<tr title="HTML Type" >';
	basicTabHtml += '<td style="padding: 4px 0;">' + 
			'<i class="fa" style="width: 9px";>&nbsp;</i>' +
			'<span style="padding-left: 6px;">HTML Type:</span></td>';
	basicTabHtml += '<td style="padding: 4px 0px 4px 8px; width: 100%;">';
	basicTabHtml += '<div ';
	basicTabHtml += 'id="semwidgckWidgetProperty_input_htmlType" ';
	basicTabHtml += 'class="semwidg-autosuggest-input" ';
	basicTabHtml += 'contentEditable="true">&nbsp;</div>';
	basicTabHtml += '</td>';
	basicTabHtml += '</tr>'; 

	for (var i = 0; i < basicProperties.length; i++) {
		var property = CKEDITOR.SemwidgJS.getProperty(type, basicProperties[i], editor);
		basicTabHtml += '<tr title="' + property[2] + '" >';
		basicTabHtml += '<td style="padding: 4px 0;">';

		if (('' + property[1]).toLowerCase() == 'query') {
			basicTabHtml += '<a href="javascript:void(0)" ' + 
					'onclick="CKEDITOR.SemwidgJS.SemwidgWidgetDialog.toggleAdditionalProperties(this, ' +
							'\'semwidgckWidgetProperty_table_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]) + '\')"' + 
				'><i class="fa fa-plus-square-o"></i></a>';
		}else {
			basicTabHtml += '<i class="fa" style="width: 9px";>&nbsp;</i>';
		}

		basicTabHtml += '<span style="padding-left: 6px;">' + property[0] + ':</span>';

		basicTabHtml += '</td>';
		basicTabHtml += '<td style="padding: 4px 0px 4px 8px; width: 100%;">';

		basicTabHtml += '<div ';
		basicTabHtml += 'id="semwidgckWidgetProperty_input_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]) + '" ';
		basicTabHtml += 'class="semwidg-autosuggest-input" ';
		basicTabHtml += 'contentEditable="true">&nbsp;</div>';

			if (('' + property[1]).toLowerCase() == 'query') {
				basicTabHtml += '<table style="width: 100%; display: none;" ' + 
					'id="semwidgckWidgetProperty_table_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]) + '">';

				for (var j = 0; j < additionalProperties.length; j++) {
					basicTabHtml += '<tr title="' + additionalProperties[j][2] + '" >';
					basicTabHtml += '<td style="padding: 4px 0;">-' + additionalProperties[j][0] + ':</td>';
					basicTabHtml += '<td style="padding: 4px 0px 4px 8px; width: 100%;">';
						basicTabHtml += '<div ';
						basicTabHtml += 'id="semwidgckWidgetProperty_input_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]) + '_' + additionalProperties[j][0] + '" ';
						basicTabHtml += 'class="semwidg-autosuggest-input" ';
						basicTabHtml += 'contentEditable="true">&nbsp;</div>';
					basicTabHtml += '</td>';
					basicTabHtml += '</tr>';
				}

				basicTabHtml += '</table>';
			}

		basicTabHtml += '</td>';
		basicTabHtml += '</tr>';
	}
	basicTabHtml += '</table>';

	basicTab.html(basicTabHtml);

	// enable Autosuggest for elements

	// html type
	var htmlTypeAutosuggestObj = CKEDITOR.SemwidgJS.textInputToAutosuggest('semwidgckWidgetProperty_input_htmlType', true);
	htmlTypeAutosuggestObj.setSelectFunction(function(value) {
		this.element.text(value);
		this.element.focus();
	});

	htmlTypeAutosuggestObj.setRequestFunction(function(value) {

		var htmlTypes = ['div', 'p', 'h1', 'h2', 'h3'];
		var resultContains = [];
		var resultContainsNot = [];

		if (htmlTypes) {
			for (var i = 0; i < htmlTypes.length; i++) {
				if (htmlTypes[i].indexOf(value) > -1) {
					resultContains.push(htmlTypes[i]);
				}else {
					resultContainsNot.push(htmlTypes[i]);
				}
			}
		};

		var simpleStringArraySort = function(a, b) {
			var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a, value);
			var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b, value);

			return levA - levB;
		};

		resultContains.sort(simpleStringArraySort);
		resultContainsNot.sort(simpleStringArraySort);

		var result = resultContains.concat(resultContainsNot);

		for (var i = 0; i < result.length; i++) {
			result[i] = ['<span style="font-family: monospace;">' + result[i] + '</span>', result[i]];
		}

		this.setResult(result);
		
	}, 0);

	// endpoint (list of configured endpoints)
	var endpointAutosuggestObj = CKEDITOR.SemwidgJS.textInputToAutosuggest('semwidgckWidgetProperty_input_endpoint', true);
	
	endpointAutosuggestObj.setSelectFunction(function(value) {
		this.element.text(value);
		this.element.focus();
	});

	endpointAutosuggestObj.setRequestFunction(function(value) {

		var endpoints = editor.getSemwidgJS().getConfig().getEndpoints();
		var resultContains = [];
		var resultContainsNot = [];

		if (endpoints) {
			for (var i = 0; i < endpoints.length; i++) {
				if (endpoints[i].getId().indexOf(value) > -1) {
					resultContains.push(endpoints[i].getId());
				}else {
					resultContainsNot.push(endpoints[i].getId());
				}
			}
		};

		var simpleStringArraySort = function(a, b) {
			var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a, value);
			var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b, value);

			return levA - levB;
		};

		resultContains.sort(simpleStringArraySort);
		resultContainsNot.sort(simpleStringArraySort);

		var result = resultContains.concat(resultContainsNot);

		for (var i = 0; i < result.length; i++) {
			result[i] = ['<span style="font-family: monospace;">' + result[i] + '</span>', result[i]];
		}

		this.setResult(result);
		
	}, 0);

	for (var i = 0; i < basicProperties.length; i++) {
		var property = CKEDITOR.SemwidgJS.getProperty(type, basicProperties[i], editor);
		if (property[1] == 'query') {
			
			var id = 'semwidgckWidgetProperty_input_' + editor.getSemwidgJS().Util.String.toCamelCase(property[0]);
			var propertyAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id, false);

			propertyAutosuggest.setSelectFunction(CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAutosuggestSelectFunction(propertyAutosuggest, editor));
			propertyAutosuggest.setRequestFunction(CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAutosuggestRequestFunction(propertyAutosuggest, editor), 0)

			propertyAutosuggest.element.on('paste', function(e) {
				CKEDITOR.SemwidgJS.SemwidgWidgetDialog.updateQueryInput($(e.currentTarget), editor);
			});

			propertyAutosuggest.element.on('keyup', function(e) {
				CKEDITOR.SemwidgJS.SemwidgWidgetDialog.updateQueryInput($(e.currentTarget), editor);
			});

			propertyAutosuggest.element.on('keydown', function(e) {
				if (e.keyCode == 32) {
					e.preventDefault();
					document.execCommand('insertHtml', false, '&nbsp;');
				}
			});

			var id2;

			id2 = id + '_Order';
			propertyAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id2, false);

			propertyAutosuggest.setSelectFunction(function(value) {
				this.element.text(value);
				this.element.focus();
			});

			propertyAutosuggest.setRequestFunction(function(value) {

				var recommendations = ['last asc', 'last desc', 'last - 1 asc', 'first + 1', 'first'];
				var resultContains = [];
				var resultContainsNot = [];

				if (recommendations) {
					for (var i = 0; i < recommendations.length; i++) {
						if (recommendations[i].indexOf(value) > -1) {
							resultContains.push(recommendations[i]);
						}else {
							resultContainsNot.push(recommendations[i]);
						}
					}
				};

				var simpleStringArraySort = function(a, b) {
					var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a, value);
					var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b, value);

					return levA - levB;
				};

				resultContains.sort(simpleStringArraySort);
				resultContainsNot.sort(simpleStringArraySort);

				var result = resultContains.concat(resultContainsNot);

				for (var i = 0; i < result.length; i++) {
					result[i] = ['<span style="font-family: monospace;">' + result[i] + '</span>', result[i]];
				}

				this.setResult(result);
				
			}, 0);	

			id2 = id + '_Endpoint';
			propertyAutosuggest = CKEDITOR.SemwidgJS.textInputToAutosuggest(id2, false);

			propertyAutosuggest.setSelectFunction(function(value) {
				this.element.text(value);
				this.element.focus();
			});

			propertyAutosuggest.setRequestFunction(function(value) {

				var endpoints = editor.getSemwidgJS().getConfig().getEndpoints();
				var resultContains = [];
				var resultContainsNot = [];

				if (endpoints) {
					for (var i = 0; i < endpoints.length; i++) {
						if (endpoints[i].getId().indexOf(value) > -1) {
							resultContains.push(endpoints[i].getId());
						}else {
							resultContainsNot.push(endpoints[i].getId());
						}
					}
				};

				var simpleStringArraySort = function(a, b) {
					var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a, value);
					var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b, value);

					return levA - levB;
				};

				resultContains.sort(simpleStringArraySort);
				resultContainsNot.sort(simpleStringArraySort);

				var result = resultContains.concat(resultContainsNot);

				for (var i = 0; i < result.length; i++) {
					result[i] = ['<span style="font-family: monospace;">' + result[i] + '</span>', result[i]];
				}

				this.setResult(result);
				
			}, 0);	
		}
	}

};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setupBasicTab = function(widget, editor) {

	if (widget) {
		editor.semwidgWidgetDialogWidget = widget;
	}else if (editor.semwidgWidgetDialogWidget) {
		widget = editor.semwidgWidgetDialogWidget;
	}else {
		return;
	}

	var type;
	if (editor.semwidgWidgetType) {
		type = editor.semwidgWidgetType;
	}else {
		type = widget.element.getAttribute('data-sw-widget');
	}

	if(!type) {
		return;
	}

	var basicProperties = CKEDITOR.SemwidgJS.getBasicWidgetProperties(editor);
	var additionalProperties = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAdditionalProperties(editor);
	var propertyName;
	var value;
	var element;
	var property;

	for (var i = 0; i < basicProperties.length; i++) {

		propertyName = editor.getSemwidgJS().Util.String.toCamelCase(basicProperties[i]);
		value = widget.element.getAttribute('data-swp-' + propertyName);

		if (value){
			element = $('#semwidgckWidgetProperty_input_' + propertyName);

			if (element.length > 0) {
				element.text(value);
			}
		}

		property = CKEDITOR.SemwidgJS.getProperty(type, basicProperties[i], editor);
		if (property[1].toLowerCase() == 'query') {

			for (var j = 0; j < additionalProperties.length; j++) {
				value = widget.element.getAttribute('data-swp-' + propertyName + '-' + additionalProperties[j][0]);
				if (value){
					element = $('#semwidgckWidgetProperty_input_' + propertyName + '_' + additionalProperties[j][0]);

					if (element.length > 0) {
						element.text(value);
					}
				}
			}
				
		}
	}

	$('#semwidgckWidgetProperty_input_htmlType').text(widget.element.$.nodeName.toLowerCase());
};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setupWidgetTab = function(widget, editor) {

	if (widget) {
		editor.semwidgWidgetDialogWidget = widget;
	}else if (editor.semwidgWidgetDialogWidget) {
		widget = editor.semwidgWidgetDialogWidget;
	}else {
		return;
	}

	var type;
	if (editor.semwidgWidgetType) {
		type = editor.semwidgWidgetType;
	}else {
		type = widget.element.getAttribute('data-sw-widget');
	}

	if(!type) {
		return;
	}

	var widgetProperties = CKEDITOR.SemwidgJS.getWidgetProperties(type, editor);
	var additionalProperties = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAdditionalProperties(editor);
	var propertyName;
	var value;
	var element;
	var property;

	for (var i = 0; i < widgetProperties.length; i++) {
		propertyName = editor.getSemwidgJS().Util.String.toCamelCase(widgetProperties[i]);

		if (CKEDITOR.SemwidgJS.SemwidgWidgetDialog.tempModel.hasOwnProperty('semwidgckWidgetProperty_input_' + propertyName)) {
			value = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.tempModel['semwidgckWidgetProperty_input_' + propertyName];
		} else {
			value = widget.element.getAttribute('data-swp-' + propertyName);
		}
		
		property = CKEDITOR.SemwidgJS.getProperty(type, widgetProperties[i], editor);

		if (value){
			element = $('#semwidgckWidgetProperty_input_' + propertyName);

			if (element.length > 0) {

				element.text(value);
				
				if (property[1].toLowerCase() == 'query') {
					CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setAutosuggestHtml(element, value, 'query', editor);
				}
			}
		}

		if (property[1].toLowerCase() == 'query') {

			for (var j = 0; j < additionalProperties.length; j++) {

				if (CKEDITOR.SemwidgJS.SemwidgWidgetDialog.tempModel.hasOwnProperty('semwidgckWidgetProperty_input_' + propertyName + '_' + additionalProperties[j][0])) {
					value = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.tempModel['semwidgckWidgetProperty_input_' + propertyName + '_' + additionalProperties[j][0]];
				} else {
					value = widget.element.getAttribute('data-swp-' + propertyName + '-' + additionalProperties[j][0]);
				}

				if (value){
					element = $('#semwidgckWidgetProperty_input_' + propertyName + '_' + additionalProperties[j][0]);

					if (element.length > 0) {
						element.text(value);
					}
				}
			}
		}
	}

};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.commitBasicTab = function(widget, editor) {
	
	if (widget) {
		editor.semwidgWidgetDialogWidget = widget;
	}else if (editor.semwidgWidgetDialogWidget) {
		widget = editor.semwidgWidgetDialogWidget;
	}else {
		return;
	}

	var type;
	if (editor.semwidgWidgetType) {
		type = editor.semwidgWidgetType;
	}else {
		type = widget.element.getAttribute('data-sw-widget');
	}

	if(!type) {
		return;
	}

	// add Id if not set
	if (editor.getSemwidgJS().Util.String.isEmpty($('#semwidgckWidgetProperty_input_id').text())) {
		$('#semwidgckWidgetProperty_input_id').text(type + '-' + editor.getSemwidgJS().Util.getUniqueId());
	}

	var basicProperties = CKEDITOR.SemwidgJS.getBasicWidgetProperties(editor);
	var additionalProperties = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAdditionalProperties(editor);
	var propertyName;
	var property;
	var value;
	for (var i = 0; i < basicProperties.length; i++) {
		propertyName = editor.getSemwidgJS().Util.String.toCamelCase(basicProperties[i]);
		value = $('#semwidgckWidgetProperty_input_' + propertyName).text();

		var old = widget.element.getAttribute('data-swp-' + propertyName);
		if (old !== value) {
			widget.element.setAttribute('data-swp-' + propertyName, value);
		}

		property = CKEDITOR.SemwidgJS.getProperty(type, basicProperties[i], editor);
		if (property[1].toLowerCase() == 'query') {

			for (var j = 0; j < additionalProperties.length; j++) {
				value = $('#semwidgckWidgetProperty_input_' + propertyName + '_' + additionalProperties[j][0]).text();
				old = widget.element.getAttribute('data-swp-' + propertyName + '-' + additionalProperties[j][0]);
				
				if (editor.getSemwidgJS().Util.String.isEmpty(value)) {
					widget.element.removeAttribute('data-swp-' + propertyName + '-' + additionalProperties[j][0]);
				}else if(old !== value) {
					widget.element.setAttribute('data-swp-' + propertyName + '-' + additionalProperties[j][0], value);
				}
			}	
		}
	}

	CKEDITOR.SemwidgJS.changeElementType(widget.element.$, 
			$('#semwidgckWidgetProperty_input_htmlType').text());

	CKEDITOR.SemwidgJS.newElementIds.push($('#semwidgckWidgetProperty_input_id').text());
	
};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.commitWidgetTab = function(widget, editor) {
	
	if (widget) {
		editor.semwidgWidgetDialogWidget = widget;
	}else if (editor.semwidgWidgetDialogWidget) {
		widget = editor.semwidgWidgetDialogWidget;
	}else {
		return;
	}

	var type;
	if (editor.semwidgWidgetType) {
		type = editor.semwidgWidgetType;
	}else {
		type = widget.element.getAttribute('data-sw-widget');
	}

	if(!type) {
		return;
	}

	var widgetProperties = CKEDITOR.SemwidgJS.getWidgetProperties(type, editor);
	var additionalProperties = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAdditionalProperties(editor);
	var propertyName;
	var property;
	var value;
	var old;
	for (var i = 0; i < widgetProperties.length; i++) {

		propertyName = editor.getSemwidgJS().Util.String.toCamelCase(widgetProperties[i]);
		value = $('#semwidgckWidgetProperty_input_' + propertyName).text();

		value = value.trim().replace(/\s/g, ' ');

		old = widget.element.getAttribute('data-swp-' + propertyName);

		if (old !== value) {
			widget.element.setAttribute('data-swp-' + propertyName, value);
		}

		property = CKEDITOR.SemwidgJS.getProperty(type, widgetProperties[i], editor);
		if (property[1].toLowerCase() == 'query') {

			for (var j = 0; j < additionalProperties.length; j++) {
				value = $('#semwidgckWidgetProperty_input_' + propertyName + '_' + additionalProperties[j][0]).text();
				old = widget.element.getAttribute('data-swp-' + propertyName + '-' + additionalProperties[j][0]);
				
				if (editor.getSemwidgJS().Util.String.isEmpty(value)) {
					widget.element.removeAttribute('data-swp-' + propertyName + '-' + additionalProperties[j][0]);
				}else if(old !== value) {
					widget.element.setAttribute('data-swp-' + propertyName + '-' + additionalProperties[j][0], value);
				}
			}	
		}
	}

	var old = widget.element.getAttribute('data-sw-widget');
	if (old !== type) {
		widget.element.setAttribute('data-sw-widget', type);
	}

};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setDefaultWidgetData = function(type) {
	if ($('#semwidgckWidgetProperty_input_styleclass').text() == '') {
		$('#semwidgckWidgetProperty_input_styleclass').text('sw-widget sw-widget-' + type);
	}
};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAutosuggestSelectFunction = function(autosuggest, editor) {
	return function(value) {

		var cursorPosition = CKEDITOR.SemwidgJS.getCursorPosition(autosuggest.element[0]);

		var text = autosuggest.element.text();

		var posStart = cursorPosition - 1;
		var posEnd = cursorPosition;

		while (!CKEDITOR.SemwidgJS.isDividingChar(text.charAt(posStart), value.charAt(posStart - 1)) && posStart >= 0) {
			posStart--;
		}
		posStart++;

		while (!CKEDITOR.SemwidgJS.isDividingChar(text.charAt(posEnd), value.charAt(posStart - 1)) && posEnd <= text.length) {
			posEnd++;
		}

		var newText = text.substr(0, posStart) + value + text.substr(posEnd);

		CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setAutosuggestHtml(autosuggest.element, newText, 'query', editor);

		var newPos = posStart + value.length;
		if (value == '{}') {
			newPos--;
		}else if (value == '{{}}') {
			newPos -= 2;
		}

		var div = $(autosuggest.element[0]);
		var child;
		var childIndex;
		var offset = 0;
		var i;
		for (i = 0; i < div.children().length; i++) {
			if (offset + $(div.children()[i]).text().length < newPos) {
				offset += $(div.children()[i]).text().length;
			}else {
				child = div.children()[i];
				childIndex = i;
				break;
			}
		}

		var classNames = child.className.split(' ');
		var type;
		for (i = 0; i < classNames.length; i++) {
			if (classNames[i].indexOf('semwidg-textinput-') == 0) {
				type = classNames[i].substr(18);
				break;
			}
		}

		// SemwidgQL
		if (type == 'semwidgql') {
			child = child.children[1];
		}
		// Text
		else if (type == 'text') {
			if (value == '{}') {
				child = div.children()[childIndex + 1].children[1];
				newPos = 1;
			}
		}	
		// or SPARQL
		else {
			if (value == '{{}}') {
				newPos++;
			}
		}

		var range = document.createRange();
		var sel = window.getSelection();

		var textNode = child.childNodes[0];

		try {

			var pos = newPos - offset;
			if (pos < 0) {
				pos = 1;
			}
			if (textNode.textContent.length < pos) {
				pos = textNode.textContent.length;
			}

			range.setStart(textNode, pos);
			range.collapse(true);
			sel.removeAllRanges();
			sel.addRange(range);
		}catch(e) {
			console.error(e);
		}	

	};
};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setAutosuggestHtml = function(element, text, mode, editor) {

	var SemwidgJS = editor.getSemwidgJS()

	var id = element.attr('id');

	var newText = '';

	if (mode.toLowerCase() == 'query') {
		var parts = SemwidgJS.Util.Widget.parseParameterParts(text);

		var lastType = 1000; 

		var semwidgqlIds = [];

		var counter = 0;
		for (var i = 0; i < parts.length; i++) {

			// SPARQL
			if (SemwidgJS.Util.String.startsWith(parts[i], '{{')) {
				if (lastType > 0) {
					counter++;
					newText += '<span class="semwidg-textinput semwidg-textinput-text">&nbsp;</span>'
				}
				newText += '<span class="semwidg-textinput semwidg-textinput-sparql">' + SemwidgJS.Util.String.htmlEscape(parts[i]) + '</span>'
				lastType = 2;
			}	
			// SemwidgQL
			else if (SemwidgJS.Util.String.startsWith(parts[i], '{')) {
				
				if (lastType > 0) {
					counter++;
					newText += '<span class="semwidg-textinput semwidg-textinput-text">&nbsp;</span>'
				}
				
				newText += '<span class="semwidg-textinput semwidg-textinput-semwidgql" ';
						newText += 'id="' + id + '-auto-' + counter + '">';
					newText += '<a class="cke_dialog_ui_button" href="javascript:void(0)" hidefocus="true" role="button" contentEditable="false" ';
							newText += 'id="' + id + '-auto-' + counter + '-edit">';
						newText += '<span class="cke_dialog_ui_button" title="Open SemwidgQL Query Editor"><i class="fa fa-search-plus"></i></span>';
					newText += '</a>';
					newText += '<span class="semwidg-textinput-semwidgql-content">' + SemwidgJS.Util.String.htmlEscape(parts[i]) + '</span>'; 
				newText += '</span>';
				
				lastType = 1;

				semwidgqlIds.push(id + '-auto-' + counter);
			}
			// text
			else {
				newText += '<span class="semwidg-textinput semwidg-textinput-text">' + SemwidgJS.Util.String.htmlEscape(parts[i]) + '</span>'
				lastType = 0;
			}

			counter++;
		}

		if (lastType > 0) {
			newText += '<span class="semwidg-textinput semwidg-textinput-text">&nbsp;</span>'
		}

		element.html(newText);

		var button;
		for (var j = 0; j < semwidgqlIds.length; j++) {
			button = $('#' + semwidgqlIds[j] + '-edit');
			button.click(function() {
				var id = this.id.substr(0, this.id.length - 5);
				// var text = $('#' + id).text();
				CKEDITOR.SemwidgJS.data.semwidgqlElement = $('#' + id);
				CKEDITOR.SemwidgJS.execCommand(editor.id, 'semwidgSemwidgQL');
				
			});
		}

	}else {

		return '<span class="semwidg-textinput semwidg-textinput-unknown">' + text + '</span>';

	}
}

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAutosuggestRequestFunction = function(autosuggest, editor) {
	return function(value) {

		var cursorPosition = CKEDITOR.SemwidgJS.getCursorPosition(autosuggest.element[0]);

		var s = '';

		var div = $(autosuggest.element[0]);
		var child;
		var offset = 0;
		var i;
		for (i = 0; i < div.children().length; i++) {
			if (offset + $(div.children()[i]).text().length < cursorPosition) {
				offset += $(div.children()[i]).text().length;
			}else {
				child = div.children()[i];
				break;
			}
		}

		if (!child) {
			var newQuerySuggestion = [];
			newQuerySuggestion.push(['(Start a new SemwidgQL query)' , '{}']);
			newQuerySuggestion.push(['(Start a new SPARQL query)' , '{{}}']);
			newQuerySuggestion.push(['(...or just type your static text)' , '']);
			autosuggest.setResult(newQuerySuggestion);
			return;
		}

		var classNames = child.className.split(' ');
		var type;
		for (i = 0; i < classNames.length; i++) {
			if (classNames[i].indexOf('semwidg-textinput-') == 0) {
				type = classNames[i].substr(18);
				break;
			}
		}

		if (type == 'text') {
			var newQuerySuggestion = [];
			newQuerySuggestion.push(['(Start a new SemwidgQL query)' , '{}']);
			newQuerySuggestion.push(['(Start a new SPARQL query)' , '{{}}']);
			newQuerySuggestion.push(['(...or just type your static text)' , '']);
			autosuggest.setResult(newQuerySuggestion);
			return;
		}
		else if(type == 'sparql') {
			var newQuerySuggestion = [];
			newQuerySuggestion.push(['Sorry, I can\'t help you with SPARQL', '']);
			autosuggest.setResult(newQuerySuggestion);
			return;
		}
		else if (type == 'semwidgql') {

			var subquery = value.substr(0, cursorPosition);
			var sortTerm = '';

			var queryStartIndex = cursorPosition;
			
			var openResources = 0;
			for (i = 0; i < subquery.length; i++) {
				if (subquery.charAt(i) == '<') {
					openResources++;
				} else if (subquery.charAt(i) == '>') {
					openResources--;
				}
			}
			if (openResources != 0) {
				autosuggest.setResult([['(Qualified Resource definition must be closed)', '>']]);
				return;
			}

			var openBraceCount = 0;
			for (i = 0; i < subquery.length; i++) {
				if (subquery.charAt(i) == '{') {
					if (openBraceCount == 0) {
						queryStartIndex = i;
					}
					openBraceCount++;
				} else if (subquery.charAt(i) == '}') {
					openBraceCount--;
					if (openBraceCount == 0) {
						queryStartIndex = cursorPosition;
					}
				}
			}

			subquery = subquery.substr(queryStartIndex + 1);

			if (subquery.indexOf('%%') >= 0 ||
				subquery.indexOf('$$') >= 0) {
				autosuggest.setResult([['(Autosuggest is not avaliable for Templates and Mappings)', '']]);
				return;
			}

			var suggestResources = false;

			// find last complete query
			for (var i = subquery.length - 1; i >= 0; i--) {

				s = subquery.substr(0, i);
				var lastChar = subquery.charAt(i);

				if (lastChar == '>') {
					s += '>.swInternal:swInternalProperty';
					break;
				}else if (lastChar == '.' && subquery.charAt(i - 1) != '\\') {
					s += '.swInternal:swInternalProperty';
					break;
				}else if (lastChar == ',') {
					s += ', swInternal:swInternalProperty';
					break;

				}else if (lastChar == '(') {
					s += '.swInternal:swInternalProperty';
					break;

				}else if (lastChar == '[') {
					s += '[swInternal:swInternalProperty';
					break;
				}else if (lastChar == '{') {

					// Subquery -> suggest resources
					suggestResources = true;
					break;
				}
			}

			if (s == '' && queryStartIndex != cursorPosition) {
				suggestResources = true;
				sortTerm = subquery.substring(i);
			}else {
				sortTerm = subquery.substring(i + 1);
			}

			if (suggestResources) {
				var resources = editor.getSemwidgJS().getConfig().getResources();
				var resourceSuggestion = [];

				resourceSuggestion.push(['*: (<span style="font-family: monospace;">*</span>) (Wildcard)' , '*']);

				for (i = 0; i < resources.length; i++) {
					resourceSuggestion.push([resources[i].getId() + ': (<span style="font-family: monospace;">' + resources[i].getId() + '</span>) (' + resources[i].getUri() + ')' , 
							resources[i].getId()]);
				}

				var posStart = cursorPosition - 1;
				var posEnd = cursorPosition - 1;

				while (!CKEDITOR.SemwidgJS.isDividingChar(value.charAt(posStart), value.charAt(posStart - 1)) && posStart >= 0) {
					posStart--;
				}
				posStart++;

				var filterValue = value.substr(posStart, posEnd);

				var simpleResourceArraySort = function(a, b) {
					var levA = CKEDITOR.SemwidgJS.levenshteinDistance(a[1], filterValue);
					var levB = CKEDITOR.SemwidgJS.levenshteinDistance(b[1], filterValue);

					return levA - levB;
				};

				resourceSuggestion.sort(simpleResourceArraySort);

				autosuggest.setResult(resourceSuggestion);
				return;
			}

			var braces = new Array();
			// close all breaces
			for (var j = 0; j < s.length; j++) {
				var currentChar = s.charAt(j);

				if (currentChar == '(') {
					braces.push(')');
				}else if (currentChar == ')') {
					braces.pop();
				}else if (currentChar == '[') {
					braces.push(']');
				}else if (currentChar == ']') {
					braces.pop();
				}else if (currentChar == '{') {
					braces.push('}');
				}else if (currentChar == '}') {
					braces.pop();
				}
			}

			for (var k = braces.length - 1; k >= 0; k--) {
				s += braces[k];
			}
			
			if (s != '') {
				CKEDITOR.SemwidgJS.SemwidgWidgetDialog.requestAutoCompleteQuery(s, autosuggest.setResult, editor, sortTerm);
			}else {
				var newQuerySuggestion = [];
				newQuerySuggestion.push(['(Start a new SemwidgQL query)' , '{}']);
				newQuerySuggestion.push(['(Start a new SPARQL query)' , '{{}}']);
				newQuerySuggestion.push(['(...or just type your static text)' , '']);
				autosuggest.setResult(newQuerySuggestion);
				return;
			}
		}
		else {

		}
	};
};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.requestAutoCompleteQuery = function(semwidgQuery, callback, editor, sortTerm) {

	var SemwidgJS = editor.getSemwidgJS();
	var SemwidgQL = editor.getSemwidgQL();

	// Create Copy of Namespaces if not exists
	CKEDITOR.SemwidgJS.SemwidgWidgetDialog.prepareAutosuggestPrefixArray(editor);

	var sparql = null;

	// create Autosuggest query
	var resources = SemwidgJS.Util.SemwidgQL.convertNamedResources(
			SemwidgJS.getConfig().getResources());

	var prefixes = SemwidgJS.Util.SemwidgQL.convertPrefixes(
			SemwidgJS.getConfig().getNamespaces());

	SemwidgQL.setCurrentModel(null);

	if (semwidgQuery) {
		semwidgQuery = semwidgQuery.trim();
	}

	try {

		var sparqlQueries = [];
		sparqlQueries.push(CKEDITOR.SemwidgJS.autosuggestPropertyQueryToSparql(semwidgQuery, editor));

		// find endpoint
		// endpoint of resource
		var endpoint;

		var possibleResource = semwidgQuery.substr(0, semwidgQuery.indexOf('.')).trim();
		var mainResource;
		if (possibleResource) {
			mainResource = SemwidgJS.getConfig().getResource(possibleResource);
		}
		if (mainResource) {
			endpoint = SemwidgJS.getConfig().getEndpoint(mainResource.getEndpoint());
		}

		// endpoint of property
		// semwidgckWidgetProperty_input_value_Endpoint
		// TODO

		// endpoint of widget
		if (!endpoint) {
			endpoint = SemwidgJS.getConfig().getEndpoint($('#semwidgckWidgetProperty_input_endpoint').val());
		}

		// default endpoint
		if (!endpoint) {
			var endpoints = SemwidgJS.getConfig().getEndpoints();
			for (var i = 0; i < endpoints.length; i++) {
				if (endpoints[i].getDefault()) {
					endpoint = endpoints[i];
					break;
				}
			}
		}

		if (!endpoint) {
			callback([['(No SPARQL endpoint was found for this query. Set an endpoint in the Basic Properties tab.)', '']]);
			return;
		}

		var url;
		if (sparqlQueries.length > 0) {

			var proxy = endpoint.getProxy();

			if (SemwidgJS.Util.String.isEmpty(proxy)) {
				proxy = 'default';
			}

			if (proxy && proxy != 'none') {

				if (proxy == 'default') {
					url = SemwidgJS.Util.getDefaultProxy() + '?url=' + endpoint.getUrl();
				}else {
					url = SemwidgJS.getConfig().getProxy(proxy).getUrl() + '?url=' + endpoint.getUrl();
				}
				
			}else {
				url = endpoint.getUrl();
			}
		}

		if (url) {

			sparql = sparqlQueries[0];

			var method = ('' + endpoint.getHttpMethod()).toUpperCase();

			if (method == 'GET') {
				if (proxy == 'none') {
					url += '?';
				}
				url += '&format=JSON';
				url += '&output=JSON';
				if (!SemwidgJS.Util.String.isEmpty(endpoint.getDefaultGraphUri())){
					url += '&default-graph-uri=' + encodeURIComponent(endpoint.getDefaultGraphUri());
				}
				url += '&query=' + encodeURIComponent(sparql);
			}

			var cacheResult = SemwidgJS.SparqlCache.getResult(
					sparql, 
					url, 
					endpoint.getDefaultGraphUri(), 
					'JSON'
				);
			
			if (cacheResult && cacheResult.getStatus() > 0 && cacheResult.getResult() != '') {

				var result = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.autosuggestSparqlJsonToObject(
						cacheResult.getResult());
				result = CKEDITOR.SemwidgJS.AutoSuggest.filterResult(result, sortTerm);
				
				var r = [];
				for (var i = 0; i < result.length; i++) {
					r.push([CKEDITOR.SemwidgJS.sanitizeHtml(result[i].getLabel()), CKEDITOR.SemwidgJS.sanitizeHtml(result[i].getValue())]);
				}
				callback(r);

			}else {

				var request;

				if (method == 'GET') {
					request = $.get(url);
				}else {
					request = $.post(
						url,
						{
							'format': 'JSON',
							'default-graph-uri': endpoint.getDefaultGraphUri(),
							'query': sparql
						}
					);
				}

				SemwidgJS.SparqlCache.insertResult(
						sparql,
						url,
						endpoint.getDefaultGraphUri(),
						'JSON',
						SemwidgJS.SparqlCache.ResultObject.STATUS_REQUESTING);

				request._swSparqlQuery = sparql;
				request._swEndpointUrl = url;
				request._swDefaultGraphUri = endpoint.getDefaultGraphUri();

				var onComplete = function(callback) {
					return function (data) {

						if (!data || data.status != 200) {
							return;
						}

						SemwidgJS.SparqlCache.insertResult(
								data._swSparqlQuery,
								data._swEndpointUrl,
								data._swDefaultGraphUri,
								'JSON',
								SemwidgJS.SparqlCache.ResultObject.STATUS_FOUND,
								data.responseText);
						var result = CKEDITOR.SemwidgJS.SemwidgWidgetDialog.autosuggestSparqlJsonToObject(
								data.responseText);
						result = CKEDITOR.SemwidgJS.AutoSuggest.filterResult(result, sortTerm);
						
						var r = [];
						for (var i = 0; i < result.length; i++) {
							r.push([CKEDITOR.SemwidgJS.sanitizeHtml(result[i].getLabel()), CKEDITOR.SemwidgJS.sanitizeHtml(result[i].getValue())]);
						}
						callback(r);
					}
				}(callback);

				request.complete(onComplete);

				callback([['loading...', '']]);
			}

		}

	}catch(e) {
		console.error(e);
	}

};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog._autosuggestPrefixArray;
CKEDITOR.SemwidgJS.SemwidgWidgetDialog.prepareAutosuggestPrefixArray = function(editor) {
	// if (!CKEDITOR.SemwidgJS.SemwidgWidgetDialog._autosuggestPrefixArray) {
		var prefixes = [];
		var namespaces = editor.getSemwidgJS().getConfig().getNamespaces();
		var i = namespaces.length;
		while(i--) {
			prefixes[i] = namespaces[i];
		}
		CKEDITOR.SemwidgJS.SemwidgWidgetDialog._autosuggestPrefixArray = prefixes;
	// }
};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.autosuggestSparqlJsonToObject = function(jsonResult, namespaces) {

	if (!jsonResult) {
		console.warn('Result is empty');
		return;
	}

	try {
		jsonResult = $.parseJSON(jsonResult);
	} catch (e) {
		console.error(e);
		return null;
	}
	
	if (jsonResult) {

		result = new Array();

		var bindings = jsonResult.results.bindings;

		var prefixes = CKEDITOR.SemwidgJS.SemwidgWidgetDialog._autosuggestPrefixArray || [];

		for (var i = 0; i < bindings.length; i++) {

			var binding = bindings[i];

			var elem = new CKEDITOR.SemwidgJS.AutoSuggest.ResultElement();
			var label = '';
			var isReversed = false;

			if (binding.hasOwnProperty('property')) {
				var p = binding.property.value;
				try {
					p = decodeURIComponent(p);
				}catch (e) {}
				elem.setValue(p);
				label += '► ';
				isReversed = false;
			}else if (binding.hasOwnProperty('propertyOf')) {
				var po = binding.propertyOf.value;
				try {
					po = decodeURIComponent(po);
				}catch (e) {}
				elem.setValue(po);
				label += '◄ ';
				isReversed = true;
			}

			//apply prefix to uri
			var val = elem.getValue();

			if (!val || val == '') {
				continue;
			}

			var prefixUri = null;
			for (var j = 0; j < prefixes.length; j++) {
				prefixUri = prefixes[j].getUri();
				if (val.toLowerCase().lastIndexOf(prefixUri.toLowerCase(), 0) === 0) {
					
					elem.setValue(prefixes[j].getId() + ':' + val.substr(prefixUri.length));

					// move prefix to top to certainly reduce search time for next iterations
					prefixes.splice(0, 0, prefixes.splice(j, 1)[0]);

					break;
				}
				prefixUri = '';
			}

			if (elem.getValue().indexOf('http') == 0) {
				elem.setValue('&lt;' + elem.getValue() + '&gt;');
			}

			if (isReversed) {
				elem.setValue('^' + elem.getValue());
			}

			if (binding.hasOwnProperty('label')) {
				var l = binding.label.value;
				try {
					l = decodeURIComponent(l);
				}catch (e) {}
				label += l + ': ';
			}else {
				if (!prefixUri || prefixUri.length < 1) {
					var lastIndex = Math.max(val.lastIndexOf('#'), val.lastIndexOf('/'));

					label += val.substr(lastIndex + 1) + ': ';

				}else {
					label += val.substr(prefixUri.length) + ': ';
				}
				
			}

			label += '(<span style="font-family: monospace;">' + elem.getValue() + '</span>) ';

			if (binding.hasOwnProperty('value')) {
				var v = binding.value.value;
				try {
					v = decodeURIComponent(v);
				}catch (e) {}
				label += '(' + v + ')';
			}

			elem.setLabel(label);

			result.push(elem);
		}

	}

	return result;
};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.getAdditionalProperties = function(editor) {
	if (!this._additionalProperties) {
		this._additionalProperties = 
			editor.getSemwidgJS().SwWidget.prototype._getQueryProperties();
	}
	
	return this._additionalProperties;
};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.toggleAdditionalProperties = function(self, id) {

	var elem = $('#' + id); 
	if (elem.is(':visible')) { 
		elem.hide();
		self.firstChild.className = 'fa fa-plus-square-o';
	} else { 
		elem.show();
		self.firstChild.className = 'fa fa-minus-square-o';
	}
};

CKEDITOR.SemwidgJS.SemwidgWidgetDialog.updateQueryInput = function(element, editor) {

	var value = element.text();

	var pos = CKEDITOR.SemwidgJS.getCursorPosition(element[0]);
	CKEDITOR.SemwidgJS.SemwidgWidgetDialog.setAutosuggestHtml(element, value, 'query', editor);

	var child;
	var childIndex;
	var offset = 0;
	var i;
	for (i = 0; i < element.children().length; i++) {
		if (offset + $(element.children()[i]).text().length < pos) {
			offset += $(element.children()[i]).text().length;
		}else {
			child = element.children()[i];
			childIndex = i;
			break;
		}
	}

	var classNames = child.className.split(' ');
	var type;
	for (i = 0; i < classNames.length; i++) {
		if (classNames[i].indexOf('semwidg-textinput-') == 0) {
			type = classNames[i].substr(18);
			break;
		}
	}

	// SemwidgQL
	if (type == 'semwidgql') {
		child = child.children[1];
	}
	// Text
	else if (type == 'text') {
		if (value == '{}') {
			child = element.children()[childIndex + 1].children[1];
			pos = 1;
		}
	}	
	// or SPARQL
	else {
		if (value == '{{}}') {
			pos++;
		}
	}

	var textLength = value.length;


	var range = document.createRange();
	var sel = window.getSelection();

	var textNode = child.childNodes[0];

	try {
		var newPos = pos - offset;
		if (textNode.textContent.length < newPos) {
			newPos = textNode.textContent.length;
		}
		range.setStart(textNode, newPos);
		range.collapse(true);
		sel.removeAllRanges();
		sel.addRange(range);
	}catch(e) {
		console.error(e);
	}
};